@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	9 April 2001
+ * Last Edited:	9 October 2001
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 2001 University of Washington.
@@ -19,6 +19,13 @@
  */
 
 
+/*				FILE TIME SEMANTICS
+ *
+ * The atime is the last read time of the file.
+ * The mtime is the last flags update time of the file.
+ * The ctime is the last write time of the file.
+ */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
@@ -452,11 +459,14 @@ long mtx_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
 
 void mtx_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 {
+  struct utimbuf times;
   struct stat sbuf;
   if (!stream->rdonly) {	/* make sure the update takes */
     fsync (LOCAL->fd);
     fstat (LOCAL->fd,&sbuf);	/* get current write time */
-    LOCAL->filetime = sbuf.st_mtime;
+    times.modtime = LOCAL->filetime = sbuf.st_mtime;
+    times.actime = time (0);		/* make sure read comes after all that */
+    utime (stream->mailbox,&times);
   }
 }
 
@@ -509,11 +519,6 @@ long mtx_ping (MAILSTREAM *stream)
       r = (mtx_parse (stream)) ? T : NIL;
       unlockfd (ld,lock);	/* release shared parse/append permission */
     }
-    else if ((sbuf.st_ctime > sbuf.st_atime)||(sbuf.st_ctime > sbuf.st_mtime)){
-      struct utimbuf times;	/* whack the times if necessary */
-      LOCAL->filetime = times.actime = times.modtime = time (0);
-      utime (stream->mailbox,&times);
-    }
   }
   return r;			/* return result of the parse */
 }
@@ -536,6 +541,7 @@ void mtx_check (MAILSTREAM *stream)
 
 void mtx_expunge (MAILSTREAM *stream)
 {
+  struct utimbuf times;
   struct stat sbuf;
   off_t pos = 0;
   int ld;
@@ -618,7 +624,9 @@ void mtx_expunge (MAILSTREAM *stream)
   else mm_log ("No messages deleted, so no update needed",(long) NIL);
   fsync (LOCAL->fd);		/* force disk update */
   fstat (LOCAL->fd,&sbuf);	/* get new write time */
-  LOCAL->filetime = sbuf.st_mtime;
+  times.modtime = LOCAL->filetime = sbuf.st_mtime;
+  times.actime = time (0);		/* reset atime to now */
+  utime (stream->mailbox,&times);
   mm_nocritical (stream);	/* release critical */
 				/* notify upper level of new mailbox size */
   mail_exists (stream,stream->nmsgs);
@@ -717,7 +725,9 @@ long mtx_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     if (!stream->rdonly) {	/* make sure the update takes */
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get current write time */
-      LOCAL->filetime = sbuf.st_mtime;
+      times.modtime = LOCAL->filetime = sbuf.st_mtime;
+      times.actime = time (0);		/* make sure atime remains greater */
+      utime (stream->mailbox,&times);
     }
   }
   return ret;
@@ -853,6 +863,7 @@ long mtx_parse (MAILSTREAM *stream)
   long curpos = LOCAL->filesize;
   long nmsgs = stream->nmsgs;
   long recent = stream->recent;
+  short added = NIL;
   short silent = stream->silent;
   fstat (LOCAL->fd,&sbuf);	/* get status */
   if (sbuf.st_size < curpos) {	/* sanity check */
@@ -892,6 +903,7 @@ long mtx_parse (MAILSTREAM *stream)
     }
     *s++ = '\0'; *t++ = '\0';	/* tie off fields */
 
+    added = T;			/* note that a new message was added */
 				/* swell the cache */
     mail_exists (stream,++nmsgs);
 				/* instantiate an elt for this message */
@@ -955,6 +967,12 @@ long mtx_parse (MAILSTREAM *stream)
   LOCAL->filesize = sbuf.st_size;
   fstat (LOCAL->fd,&sbuf);	/* get status again to ensure time is right */
   LOCAL->filetime = sbuf.st_mtime;
+  if (added) {			/* make sure atime updated */
+    struct utimbuf times;
+    times.actime = time (0);
+    times.modtime = LOCAL->filetime;
+    utime (stream->mailbox,&times);
+  }
   stream->silent = silent;	/* can pass up events now */
   mail_exists (stream,nmsgs);	/* notify upper level of new mailbox size */
   mail_recent (stream,recent);	/* and of change in recent messages */
@@ -1028,8 +1046,9 @@ void mtx_read_flags (MAILSTREAM *stream,MESSAGECACHE *elt)
 
 void mtx_update_status (MAILSTREAM *stream,unsigned long msgno,long syncflag)
 {
-  MESSAGECACHE *elt = mail_elt (stream,msgno);
+  struct utimbuf times;
   struct stat sbuf;
+  MESSAGECACHE *elt = mail_elt (stream,msgno);
   unsigned long j,k = 0;
 				/* readonly */
   if (stream->rdonly || !elt->valid) mtx_read_flags (stream,elt);
@@ -1053,7 +1072,9 @@ void mtx_update_status (MAILSTREAM *stream,unsigned long msgno,long syncflag)
     if (syncflag) {		/* sync if requested */
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get new write time */
-      LOCAL->filetime = sbuf.st_mtime;
+      times.modtime = LOCAL->filetime = sbuf.st_mtime;
+      times.actime = time (0);		/* make sure read is later */
+      utime (stream->mailbox,&times);
     }
   }
 }
