@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: attach.c,v 4.63 1999/05/27 23:09:27 mikes Exp $";
+static char rcsid[] = "$Id: attach.c,v 4.65 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Routines to support attachments in the Pine composer 
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: attach.c,v 4.63 1999/05/27 23:09:27 mikes Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -38,14 +38,14 @@ static char rcsid[] = "$Id: attach.c,v 4.63 1999/05/27 23:09:27 mikes Exp $";
 		       int,char *,char *,int,int *);
     PATMT *NewAttach(char *, long, char *);
     void   ZotAttach(struct pico_atmt *);
-    int    sinserts(char *, int, char *, int);
+    void   sinserts(char *, int, char *, int);
     int	   AttachUpload(char *, char *);
     int	   AttachCancel(char *);
 #else
     int    ParseAttach();
     PATMT *NewAttach();
     void   ZotAttach();
-    int    sinserts();
+    void   sinserts();
     int	   AttachUpload();
     int	   AttachCancel();
 #endif
@@ -193,7 +193,7 @@ char *fn, *sz, *cmnt;
 			    ? "."
 			    : ((gmode & MDTREE) || opertree[0])
 				 ? opertree : gethomedir(NULL));
-	    if((fbrv = FileBrowse(fn, NLINE, bfn, NLINE, sz, FB_READ)) == 1){
+	    if((fbrv = FileBrowse(fn, NLINE, bfn, NLINE, sz, FB_READ | FB_ATTACH)) == 1){
 	      if ((strlen(fn)+strlen(S_FILESEP)+strlen(bfn)) < NLINE){
 		strcat(fn, S_FILESEP);
 		strcat(fn, bfn);
@@ -309,6 +309,8 @@ char *fn, *sz, *cmnt;
 	    break;
 	}
     }
+
+    return(0);
 }
 
 
@@ -1222,9 +1224,10 @@ off_t l;
 /*
  * sinserts - s insert into another string
  */
+void
 sinserts(ds, dl, ss, sl)
-char *ds, *ss;					/* dest. and source strings */
-int  dl, sl;					/* their lengths */
+    char *ds, *ss;				/* dest. and source strings */
+    int   dl, sl;				/* their lengths */
 {
     char *dp, *edp;				/* pointers into dest. */
     int  j;					/* jump difference */
