@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: basic.c,v 4.31 2001/06/21 17:49:51 hubert Exp $";
+static char rcsid[] = "$Id: basic.c,v 4.33 2001/10/18 20:13:04 jpf Exp $";
 #endif
 /*
  * Program:	Cursor manipulation functions
@@ -267,6 +267,7 @@ gotobop(f, n)
 int f, n;	/* default Flag & Numeric argument */
 {
     int quoted, qlen;
+    char qstr[NLINE], qstr2[NLINE];
 
     if (n < 0)	/* the other way...*/
       return(gotoeop(f, -n));
@@ -285,14 +286,15 @@ int f, n;	/* default Flag & Numeric argument */
 	 * PLUS: if there's a quote string, a quoted-to-non-quoted
 	 *	 line transition.
 	 */
-	quoted = (Pmaster && Pmaster->quote_str
-		  && quote_match(Pmaster->quote_str, curwp->w_dotp));
-	qlen   = quoted ? strlen(Pmaster->quote_str) : 0;
+	quoted = (Pmaster && Pmaster->quote_str)
+	  ? quote_match(Pmaster->quote_str, curwp->w_dotp, qstr, NLINE) : 0;
+	qlen   = quoted ? strlen(qstr) : 0;
 	while(lback(curwp->w_dotp) != curbp->b_linep
 	      && llength(lback(curwp->w_dotp)) > qlen
 	      && ((Pmaster && Pmaster->quote_str)
-		    ? quoted == quote_match(Pmaster->quote_str,
-					    lback(curwp->w_dotp))
+		    ? (quoted == quote_match(Pmaster->quote_str,
+					    lback(curwp->w_dotp), qstr2, NLINE)
+		       && !strcmp(qstr, qstr2))
 		    : 1)
 	      && lgetc(curwp->w_dotp, qlen).c != TAB
 	      && lgetc(curwp->w_dotp, qlen).c != ' ')
@@ -335,6 +337,7 @@ int f, n;	/* default Flag & Numeric argument */
 
 {
     int quoted, qlen;
+    char qstr[NLINE], qstr2[NLINE];
 
     if (n < 0)	/* the other way...*/
       return(gotobop(f, -n));
@@ -353,15 +356,16 @@ int f, n;	/* default Flag & Numeric argument */
 	 * PLUS: if there's a quote string, a quoted-to-non-quoted
 	 *	 line transition.
 	 */
-	quoted = (Pmaster && Pmaster->quote_str
-		  && quote_match(Pmaster->quote_str, curwp->w_dotp));
-	qlen   = quoted ? strlen(Pmaster->quote_str) : 0;
+	quoted = (Pmaster && Pmaster->quote_str)
+		  ? quote_match(Pmaster->quote_str, curwp->w_dotp, qstr, NLINE) : 0;
+	qlen   = quoted ? strlen(qstr) : 0;
 	
 	while(curwp->w_dotp != curbp->b_linep
 	      && llength(lforw(curwp->w_dotp)) > qlen
 	      && ((Pmaster && Pmaster->quote_str)
-		   ? quoted == quote_match(Pmaster->quote_str,
-					   lforw(curwp->w_dotp))
+		   ? (quoted == quote_match(Pmaster->quote_str,
+					   lforw(curwp->w_dotp), qstr2, NLINE)
+		      && !strcmp(qstr, qstr2))
 		   : 1)
 	      && lgetc(lforw(curwp->w_dotp), qlen).c != TAB
 	      && lgetc(lforw(curwp->w_dotp), qlen).c != ' ')
