@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: bind.c,v 4.22 1998/08/27 23:01:21 hubert Exp $";
+static char rcsid[] = "$Id: bind.c,v 4.23 2000/11/09 21:53:43 hubert Exp $";
 #endif
 /*
  * Program:	Key binding routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: bind.c,v 4.22 1998/08/27 23:01:21 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -181,6 +181,7 @@ whelp(f, n)
     }
 
     sgarbf = TRUE;
+    return(FALSE);
 }
 
 static KEYMENU menu_scroll[] = {
@@ -377,6 +378,7 @@ int list[][2];
 /*
  * rebind - replace the first function with the second
  */
+void
 rebindfunc(a, b)
     int (*a) PROTO((int, int)), (*b) PROTO((int, int));
 {
