@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.85 2001/06/21 17:49:51 hubert Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.86 2001/10/08 21:23:49 jpf Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -92,7 +92,7 @@ static	char	*browser_title = NULL;
 
 
 #ifdef	ANSI
-    struct bmaster *getfcells(char *);
+    struct bmaster *getfcells(char *, int);
     int    PaintCell(int, int, int, struct fcell *, int);
     int    PaintBrowser(struct bmaster *, int, int *, int *);
     void   BrowserKeys(void);
@@ -318,9 +318,9 @@ pico_file_browse(pdata, dir, dirlen, fn, fnlen, sz, flags)
  *                   0 if no files seleted
  *                  -1 if there where problems
  */
-FileBrowse(dir, dirlen, fn, fnlen, sz, flags)
+FileBrowse(dir, dirlen, fn, fnlen, sz, fb_flags)
 char *dir, *fn, *sz;			/* dir, name and optional size */
-int   dirlen, fnlen, flags;
+int   dirlen, fnlen, fb_flags;
 {
     int status, i, j, c, new_c;
     int row, col, crow, ccol;
@@ -345,7 +345,7 @@ int   dirlen, fnlen, flags;
     }
 
     /* build contents of cell structures */
-    if((gmp = getfcells(dir)) == NULL)
+    if((gmp = getfcells(dir, fb_flags)) == NULL)
       return(-1);
     
     tp = NULL;
@@ -944,7 +944,7 @@ int   dirlen, fnlen, flags;
 		    }
  
 		    if(isdir(child, (long *) NULL, NULL)){
-			if((mp = getfcells(child)) == NULL){
+			if((mp = getfcells(child, fb_flags)) == NULL){
 			    /* getfcells should explain what happened */
 			    i++;
 			    break;
@@ -1062,7 +1062,7 @@ int   dirlen, fnlen, flags;
 			 * and redraw...
 			 */
 			if(!strcmp(tmp, gmp->dname)){ 
-			    if((mp = getfcells(gmp->dname)) == NULL)
+			    if((mp = getfcells(gmp->dname, fb_flags)) == NULL)
 			      /* getfcells should explain what happened */
 			      break;
 
@@ -1184,7 +1184,7 @@ int   dirlen, fnlen, flags;
 			 */
 			if(!strcmp(tmp, gmp->dname)){ 
 			    strcpy(child, gmp->current->fname);
-			    if((mp = getfcells(gmp->dname)) == NULL)
+			    if((mp = getfcells(gmp->dname, fb_flags)) == NULL)
 			      /* getfcells should explain what happened */
 			      break;
 
@@ -1288,7 +1288,7 @@ int   dirlen, fnlen, flags;
 			    *p = '\0';
 			    strcpy(tmp, (p == child) ? S_FILESEP: child);
 
-			    if((mp = getfcells(tmp)) == NULL)
+			    if((mp = getfcells(tmp, fb_flags)) == NULL)
 			      /* getfcells should explain what happened */
 			      break;
 
@@ -1364,13 +1364,21 @@ int   dirlen, fnlen, flags;
 			  }
 		    }
 		}
+		else if((fb_flags&FB_SAVE) && p[0] == '.' && p[1] == '\0'){
+		    if ((strlen(gmp->dname) < dirlen) && 
+			(strlen(gmp->current->fname) < fnlen)){
+			strcpy(dir, gmp->dname);
+		    }
+		    zotmaster(&gmp);
+		    return(0);  /* just change the directory, still return no selection */
+		}
 		else{
 		    if(tmp[1] != '\0')		/* were in root? */
 		      strcat(tmp, S_FILESEP);
 		    strcat(tmp, gmp->current->fname);
 		}
 
-		if((mp = getfcells(tmp)) == NULL)
+		if((mp = getfcells(tmp, fb_flags)) == NULL)
 		  /* getfcells should explain what happened */
 		  break;
 
@@ -1552,8 +1560,9 @@ int   dirlen, fnlen, flags;
  *             return NULL if there's a problem.
  */
 struct bmaster *
-getfcells(dname)
+getfcells(dname, fb_flags)
     char *dname;
+    int   fb_flags;
 {
     int  i, 					/* various return codes */
          flength,
@@ -1627,7 +1636,8 @@ getfcells(dname)
 	 * Filter dot files?  Always filter ".", never filter "..",
 	 * and sometimes fitler ".*"...
 	 */
-	if(*np == '.' && !(*(np+1) == '.' && *(np+2) == '\0')
+	if(*np == '.' && (!(*(np+1) == '.' && *(np+2) == '\0')
+			  && !(*(np+1) == '\0' && (fb_flags&FB_SAVE)))
 	   && (*(np+1) == '\0' || !(gmode & MDDOTSOK))){
 	    np += strlen(np) + 1;
 	    continue;
@@ -1706,7 +1716,9 @@ getfcells(dname)
 	    ncp->mode = FIODIR;
 	    sprintf(ncp->size, "(%sdir)",
 		    (ncp->fname[0] == '.' && ncp->fname[1] == '.'
-		     && ncp->fname[2] == '\0') ? "parent " : "");
+		     && ncp->fname[2] == '\0') ? "parent " : 
+		    ((fb_flags&FB_SAVE) && ncp->fname[0] == '.' && ncp->fname[1] == '\0'
+		     ? "SELECT " : ""));
 	    break;
 	  case FIOSYM :
 	    ncp->mode = FIOSYM;
@@ -2213,7 +2225,7 @@ BrowserRunChild(child, dir)
     if(t_in && isdir(dir, NULL, &t_out) && t_in < t_out){
 	struct bmaster *mp;
 
-	if(mp = getfcells(dir)){
+	if(mp = getfcells(dir, 0)){
 	    zotmaster(&gmp);
 	    gmp = mp;
 	}
