@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.102 2001/06/21 17:49:51 hubert Exp $";
+static char rcsid[] = "$Id: display.c,v 4.105 2001/11/14 01:17:11 jpf Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -1392,7 +1392,7 @@ EXTRAKEYS *extras;
 
     (*term.t_rev)(1);
 
-    maxl = (nbuf < term.t_ncol - plen - 1) ? nbuf : term.t_ncol - plen - 1;
+    maxl = (nbuf-1 < term.t_ncol - plen - 1) ? nbuf-1 : term.t_ncol - plen - 1;
 
     pputs(buf, 1);
     b = &buf[(flg & QBOBUF) ? 0 : strlen(buf)];
@@ -1654,7 +1654,7 @@ void	*arg;
 	switch(ap[1]){
 	  case '%':
 	  case 'c':
-	    l += 1;
+	    l += arg ? 2 : 1;
 	    break;
 	  case 'd':
 	    l += (long)dumbroot((int)arg, 10);
@@ -1669,7 +1669,7 @@ void	*arg;
 	    l += (long)dumbroot((int)arg, 16);
 	    break;
 	  case 's':
-            l += strlen((char *)arg);
+            l += arg ? strlen((char *)arg) : 2;
 	    break;
 	}
     }
@@ -1687,7 +1687,11 @@ void	*arg;
 	else if(*bufp == '%'){
 	    switch(*++bufp){
 	      case 'c':
-		pputc((char)(int)arg, 0);
+		if(arg)
+		  pputc((char)(int)arg, 0);
+		else {
+		    pputs("%c", 0);
+		}
 		break;
 	      case 'd':
 		mlputi((int)arg, 10);
@@ -1702,7 +1706,7 @@ void	*arg;
 		mlputi((int)arg, 8);
 		break;
 	      case 's':
-		pputs((char *)arg, 0);
+		pputs(arg ? (char *)arg : "%s", 0);
 		break;
 	      case '%':
 	      default:
@@ -2450,7 +2454,8 @@ void
 wkeyhelp(keymenu)
 KEYMENU *keymenu;
 {
-    char *obufp, *p, fkey[4], linebuf[NLINE];
+    char *obufp, *p, fkey[4];
+    char  linebuf[2*NLINE];	/* "2" is for space for invert tokens */
     int   row, slot, tspace, adjusted_tspace, nspace[6], index, n;
 #ifdef	MOUSE
     char  nbuf[NLINE];
