@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.86 2001/07/13 19:39:27 hubert Exp $
+ * $Id: efunc.h,v 4.88 2001/10/30 18:40:31 hubert Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -267,6 +267,8 @@ extern	void pico_set_nbg_color PROTO((void));
 extern	void pico_set_normal_color PROTO((void));
 extern	int pico_usingcolor PROTO((void));
 extern	char *color_to_asciirgb PROTO((char *));
+extern	char *colorx PROTO((int));
+extern	char *color_to_canonical_name PROTO((char *));
 extern	int pico_count_in_color_table PROTO((void));
 /* these color functions aren't needed for Windows */
 extern	int pico_hascolor PROTO((void));
@@ -305,7 +307,6 @@ extern	int forwdel PROTO((int, int));
 extern	int backdel PROTO((int, int));
 extern	int killtext PROTO((int, int));
 extern	int yank PROTO((int, int));
-extern	char *colorx PROTO((int));
 extern	COLOR_PAIR *pico_apply_rev_color PROTO((COLOR_PAIR *, int));
 
 /* region.c */
@@ -334,6 +335,6 @@ extern	int backword PROTO((int, int));
 extern	int forwword PROTO((int, int));
 extern	int fillpara PROTO((int, int));
 extern	int inword PROTO((void));
-extern	int quote_match PROTO((char *, LINE *));
+extern	int quote_match PROTO((char *, LINE *, char *, int));
 
 #endif	/* EFUNC_H */
