@@ -1,5 +1,5 @@
 /*
- * $Id: estruct.h,v 4.51 2000/06/26 21:39:18 hubert Exp $
+ * $Id: estruct.h,v 4.53 2001/07/19 17:08:51 hubert Exp $
  *
  * Program:	Struct and preprocessor definitions
  *
@@ -323,6 +323,14 @@ typedef struct COLOR_PAIR {
 #define COLOR_ANSI8_OPT		0x01
 #define COLOR_ANSI16_OPT	0x02
 
+/* styles for pico_apply_rev_color() */
+#define IND_COL_FLIP		0
+#define IND_COL_REV		1
+#define IND_COL_FG		2
+#define IND_COL_FG_NOAMBIG	3
+#define IND_COL_BG		4
+#define IND_COL_BG_NOAMBIG	5
+
 
 /*
  * All text is kept in circularly linked lists of "LINE" structures. These
