@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.74 2001/06/21 17:49:51 hubert Exp $";
+static char rcsid[] = "$Id: file.c,v 4.75 2001/10/08 21:23:49 jpf Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -475,7 +475,7 @@ int f, n;
         register WINDOW *wp;
         register int    s;
         char            fname[NFILEN];
-	char		shows[NLINE], *bufp;
+	char		shows[NLINE], origshows[NLINE], *bufp;
 	EXTRAKEYS	menu_write[3];
 
 	if(curbp->b_fname[0] != 0)
@@ -587,6 +587,7 @@ int f, n;
 		  strcpy(shows, ((gmode & MDTREE) || opertree[0])
 				   ? opertree : gethomedir(NULL));
 
+		strcpy(origshows, shows);
 		if ((s = FileBrowse(shows, NLINE, fname, NFILEN, NULL, FB_SAVE)) == 1) {
 		  if (strlen(shows)+strlen(S_FILESEP)+strlen(fname) < NLINE){
 		    strcat(shows, S_FILESEP);
@@ -598,6 +599,11 @@ int f, n;
 		    sleep(3);
 		  }
 		}
+		else if (s == 0 && strcmp(shows, origshows)){
+		    strcat(shows, S_FILESEP);
+		    strcat(shows, fname);
+		    strcpy(fname, shows);
+		}
 		else if (s == -1){
 		  emlwrite("Cannot write. File name too long!!",NULL);
 		  sleep(3);
