@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: line.c,v 4.23 2001/06/21 17:49:51 hubert Exp $";
+static char rcsid[] = "$Id: line.c,v 4.25 2001/10/18 20:13:04 jpf Exp $";
 #endif
 /*
  * Program:	Line management routines
@@ -616,9 +616,10 @@ lisblank(line)
     LINE *line;
 {
     int n = 0;
+    char qstr[NLINE];
 
-    n = (Pmaster && Pmaster->quote_str && quote_match(Pmaster->quote_str,line))
-	  ? strlen(Pmaster->quote_str) : 0;
+    n = (Pmaster && Pmaster->quote_str && quote_match(Pmaster->quote_str,line,qstr,NLINE))
+	  ? strlen(qstr) : 0;
 
     for(; n < llength(line); n++)
       if(!isspace((unsigned char) lgetc(line, n).c))
