@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: main.c,v 4.65 2000/05/19 17:42:43 hubert Exp $";
+static char rcsid[] = "$Id: main.c,v 4.66 2001/06/22 23:42:14 hubert Exp $";
 #endif
 /*
  * Program:	Main stand-alone Pine Composer routines
@@ -75,6 +75,7 @@ static int fkm[12][2] = {
 
 char *pico_args PROTO((int, char **, int *, int *));
 void  pico_args_help PROTO((void));
+void  pico_vers_help PROTO((void));
 void  pico_display_args_err PROTO((char *, char **, int));
 
 char  args_pico_missing_flag[]  = "unknown flag \"%c\"";
@@ -113,6 +114,7 @@ char *args_pico_args[] = {
 #endif
 "\t +[line#] \tLine - start on line# line, default=1",
 "\t -v \t\tView - view file",
+"\t -version\tPico version number",
 "", 
 "\t All arguments may be followed by a file name to display.",
 "",
@@ -378,8 +380,11 @@ Loop:
       /* while more chars in this argument */
       else while(*++*av){
 
+	if(strcmp(*av, "version") == 0){
+	    pico_vers_help();
+	}
 #if	defined(DOS) || defined(OS2)
-	if(strcmp(*av, "cnf") == 0
+	else if(strcmp(*av, "cnf") == 0
 	   || strcmp(*av, "cnb") == 0
 	   || strcmp(*av, "crf") == 0
 	   || strcmp(*av, "crb") == 0){
@@ -608,6 +613,21 @@ pico_args_help()
 }
 
 
+void
+pico_vers_help()
+{
+    char v0[100];
+    char *v[2];
+
+    sprintf(v0, "Pico %.50s", version);
+    v[0] = v0;
+    v[1] = NULL;
+
+    pico_display_args_err(NULL, v, 0);
+    exit(1);
+}
+
+
 /*----------------------------------------------------------------------
    write argument error to the display...
 
