@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pico.c,v 4.124 2000/11/09 21:53:43 hubert Exp $";
+static char rcsid[] = "$Id: pico.c,v 4.125 2001/06/21 17:49:51 hubert Exp $";
 #endif
 /*
  * Program:	Main Pine Composer routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: pico.c,v 4.124 2000/11/09 21:53:43 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -123,8 +123,11 @@ PICO *pm;
     register int    n;
     char     bname[NBUFN];		/* buffer name of file to read */
     extern   struct on_display ods;
-    int      checkpointcnt = 0, input = 0, cursor_shown;
+    int      checkpointcnt = 0, input = 0;
     char     chkptfile[NLINE];
+#ifdef	_WINDOWS
+    int      cursor_shown;
+#endif
 
     Pmaster       = pm;
     gmode	  = MDWRAP;
@@ -483,7 +486,6 @@ int c, f, n;
 {
     register KEYTAB *ktp;
     register int    status;
-    register int    i;
 
     ktp = (Pmaster) ? &keytab[0] : &pkeytab[0];
 
@@ -541,7 +543,7 @@ int c, f, n;
 	    register int j;
 	    register int k;
 
-	    for(i = j = k = 0; j < llength(curwp->w_dotp); j++, k++)
+	    for(j = k = 0; j < llength(curwp->w_dotp); j++, k++)
 	      if(isspace((unsigned char)lgetc(curwp->w_dotp, j).c)){
 		  if(lgetc(curwp->w_dotp, j).c == TAB)
 		    while(k+1 & 0x07)
@@ -669,7 +671,7 @@ int f, n;
     register int    s;
 
     if(Pmaster){
-	char *prompt, *result;
+	char *result;
 
 	/* First, make sure there are no outstanding problems */ 
 	if(AttachError()){
