@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: random.c,v 4.22 2001/07/19 17:08:51 hubert Exp $";
+static char rcsid[] = "$Id: random.c,v 4.23 2001/10/30 18:40:31 hubert Exp $";
 #endif
 /*
  * Program:	Random routines
@@ -379,54 +379,6 @@ int f, n;
 }
 
 
-/*
- * Returns a pointer to the name of color number color.
- * The colors 0 through 7 are the names of the ANSI colors, then
- * come the strings color008, color009, color010, ...
- *
- * Warning, returns a pointer to a static buffer, so the result must be
- * copied before another call to colorx.
- */
-char *
-colorx(color)
-    int color;
-{
-    static char cbuf[12];
-
-    switch(color){
-      case COL_BLACK:
-	strcpy(cbuf, "black");
-	break;
-      case COL_RED:
-	strcpy(cbuf, "red");
-	break;
-      case COL_GREEN:
-	strcpy(cbuf, "green");
-	break;
-      case COL_YELLOW:
-	strcpy(cbuf, "yellow");
-	break;
-      case COL_BLUE:
-	strcpy(cbuf, "blue");
-	break;
-      case COL_MAGENTA:
-	strcpy(cbuf, "magenta");
-	break;
-      case COL_CYAN:
-	strcpy(cbuf, "cyan");
-	break;
-      case COL_WHITE:
-	strcpy(cbuf, "white");
-	break;
-      default:
-        sprintf(cbuf, "color%03.3d", color);
-	break;
-    }
-
-    return(cbuf);
-}
-
-
 /*
  * Original idea from Stephen Casner <casner@acm.org>.
  *
