@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: search.c,v 4.25 2000/11/09 21:53:43 hubert Exp $";
+static char rcsid[] = "$Id: search.c,v 4.26 2001/06/21 17:49:51 hubert Exp $";
 #endif
 /*
  * Program:	Searching routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: search.c,v 4.25 2000/11/09 21:53:43 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2001 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -312,7 +312,7 @@ int  *wrapt;
   register         int status;
   char             lpat[NPAT], origpat[NPAT];     /* case sensitive pattern */
   EXTRAKEYS        menu_pat[2];
-  int              repl_all = FALSE, unhl = TRUE;
+  int              repl_all = FALSE;
   char             prompt[2*NLINE+32];
 
     forscan(wrapt, defpat, NULL, 0, PTBEG);    /* go to word to be replaced */
@@ -442,7 +442,7 @@ char *repl;
 {
   register         int status = 0;
   char             prompt[NLINE], realpat[NPAT];
-  int              wrapt, wraplatch = 0, n = 0;
+  int              wrapt, n = 0;
   LINE		  *stop_line   = curwp->w_dotp;
   int		   stop_offset = curwp->w_doto;
 
@@ -573,7 +573,6 @@ int   repl_mode;
 readnumpat(prompt)
 char *prompt;
 {
-    register int s;
     int		 i, n;
     char	 tpat[NPAT+20];
     EXTRAKEYS    menu_pat[2];
