@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: word.c,v 4.33 2000/05/16 22:56:33 jpf Exp $";
+static char rcsid[] = "$Id: word.c,v 4.36 2001/10/18 20:13:04 jpf Exp $";
 #endif
 /*
  * Program:	Word at a time routines
@@ -362,19 +362,40 @@ inword()
 
 
 /*
- * Return TRUE if whatever starts the line matches the quote string
+ * Return number of quotes if whatever starts the line matches the quote string
  */
-quote_match(q, l)
+quote_match(q, l, buf, buflen)
     char *q;
     LINE *l;
+    char *buf;
+    int   buflen;
 {
-    register int i;
+    register int i, n, j, qb;
 
-    for(i = 0; i <= llength(l) && q[i]; i++)
-      if(q[i] != lgetc(l, i).c)
-	return(0);
+    if(*q == '\0')
+      return(1);
 
-    return(1);
+    qb = q[strlen(q)-1] == ' ' ? 1 : 0;
+    *buf = '\0';
+    for(n = 0, j = 0; ;){
+	for(i = 0; j <= llength(l) && qb ? q[i+1] : q[i]; i++, j++)
+	  if(q[i] != lgetc(l, j).c)
+	    return(n);
+
+	n++;
+	if((!qb && q[i] == '\0') || (qb && q[i+1] == '\0')){
+	    if(strlen(buf) + strlen(q) + 1 < buflen){
+		strcat(buf,q);
+		if(qb && (j > llength(l) || lgetc(l, j).c != ' '))
+		  buf[strlen(buf)-1] = '\0';
+	    }
+	}
+	if(j > llength(l))
+	  return(n);
+	else if(!qb || lgetc(l, j).c == ' ')
+	  j++;
+    }
+    return(n);  /* never reached */
 }
 
 
@@ -385,8 +406,8 @@ int f, n;	/* deFault flag and Numeric argument */
 
 {
     int	    i, j, c, qlen, word[NSTRING], same_word,
-	    spaces, word_len, line_len, line_last;
-    char   *qstr;
+	    spaces, word_len, line_len, line_last, qn;
+    char   *qstr, qstr2[NSTRING];
     LINE   *eopline;
     REGION  region;
 
@@ -411,8 +432,8 @@ int f, n;	/* deFault flag and Numeric argument */
 
     /* determine if we're justifying quoted text or not */
     qstr = (Pmaster && Pmaster->quote_str
-	      && quote_match(Pmaster->quote_str, curwp->w_dotp))
-	     ? Pmaster->quote_str : NULL;
+	    && quote_match(Pmaster->quote_str, curwp->w_dotp, qstr2, NSTRING)
+	    && *qstr2) ? qstr2 : NULL;
     qlen = qstr ? strlen(qstr) : 0;
 
     /* let yank() know that it may be restoring a paragraph */
@@ -462,7 +483,7 @@ int f, n;	/* deFault flag and Numeric argument */
 	  default :
 	    if(spaces){				/* flush word? */
 		if(line_len
-		   && line_len + word_len + qlen + 1 > fillcol
+		   && line_len + word_len + 1 > fillcol
 		   && (line_len = fpnewline(qstr)))
 		  line_last = ' ';	/* no word-flush space! */
 
@@ -506,7 +527,7 @@ int f, n;	/* deFault flag and Numeric argument */
     }
 
     if(word_len){
-	if(line_len + word_len + qlen + 1 > fillcol)
+	if(line_len + word_len + 1 > fillcol)
 	  (void) fpnewline(qstr);
 	else if(line_len && !isspace((unsigned char) line_last))
 	  linsert(1, ' ');
