@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.107 2001/06/22 23:44:34 hubert Exp $";
+static char rcsid[] = "$Id: args.c,v 4.112 2001/11/28 19:12:44 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -70,6 +70,8 @@ void args_add_attach PROTO((PATMT **, char *, int));
  */
 char args_err_missing_pinerc[] =	"missing argument for option \"-pinerc\" (use - for standard out)";
 char args_err_missing_aux[] =		"missing argument for option \"-aux\"";
+char args_err_missing_passfile[] =	"missing argument for option \"-passfile\"";
+char args_err_non_abs_passfile[] =	"argument to \"-passfile\" should be fully-qualified";
 char args_err_missing_lu[] =		"missing argument for option \"-create_lu\"\nUsage: pine -create_lu <addrbook_file> <addrbook_sort_type>";
 char args_err_missing_sort[] =		"missing argument for option \"-sort\"";
 char args_err_missing_flag_arg[] =	"missing argument for flag \"%c\"";
@@ -104,7 +106,7 @@ char *args_pine_args[] = {
 " -bail\t\tExit if pinerc file doesn't already exist",
 #ifdef	DEBUG
 " -d n\t\tDebug - set debug level to 'n', or use the following:",
-" -d keywords...\tflush,timestamp,imap=0..4,numfiles=0..31,verbose=0..9",
+" -d keywords...\tflush,timestamp,imap=0..4,tcp,numfiles=0..31,verbose=0..9",
 #endif
 " -f <folder>\tFolder - give folder name to open",
 " -c <number>\tContext - which context to apply to -f arg",
@@ -131,6 +133,10 @@ char *args_pine_args[] = {
 #else
 " -aux <aux_files_dir>\tUse this with remote pinerc",
 " -P <pine.conf>\tUse pine.conf file for default settings",
+#ifdef PASSFILE
+" -passfile <fully_qualified_filename>\tSet the password file to something other",
+"\t\tthan the default",
+#endif /* PASSFILE */
 #endif
 " -x <config>\tUse configuration exceptions in <config>.",
 "\t\tExceptions are used to override your default pinerc",
@@ -271,6 +277,31 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+#ifdef PASSFILE
+	      else if(strcmp(*av, "passfile") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL){
+			  if(!is_absolute_path(str)){
+			      display_args_err(args_err_non_abs_passfile,
+					       NULL, 1);
+			      ++usage;
+			  }
+			  else{
+			      if(pine_state->passfile)
+				fs_give((void **)&pine_state->passfile);
+
+			      pine_state->passfile = cpystr(str);
+			  }
+		      }
+		  }
+		  else{
+		      display_args_err(args_err_missing_passfile, NULL, 1);
+		      ++usage;
+		  }
+
+		  goto Loop;
+	      }
+#endif  /* PASSFILE */
 #endif
 	      else if(strcmp(*av, "create_lu") == 0){
 		  if(ac > 2){
@@ -832,12 +863,15 @@ process_debug_str(debug_str)
 		    }
 		    else{
 			i = atoi(q+1);
-			ps_global->debug_imap = min(4,max(0,i));
+			ps_global->debug_imap = min(5,max(0,i));
 		    }
 		}
 		else if(struncmp(*p, "flush", 5) == 0){
 		    ps_global->debug_flush = 1;
 		}
+		else if(struncmp(*p, "tcp", 3) == 0){
+		    ps_global->debug_tcp = 1;
+		}
 		else if(struncmp(*p, "verbose", 7) == 0){
 		    q = *p + 7;
 		    if(!*q || !*(q+1) || *q != '=' ||
@@ -902,10 +936,10 @@ process_debug_str(debug_str)
 	}
 	else{
 	    debug = atoi(debug_str);
-	    if(debug > 7){
-		ps_global->debug_imap = 4;
-		ps_global->debug_timestamp = 1;
-	    }
+	    if(debug > 9)
+	      ps_global->debug_imap = 5;
+	    else if(debug > 7)
+	      ps_global->debug_imap = 4;
 	    else if(debug > 6)
 	      ps_global->debug_imap = 3;
 	    else if(debug > 4)
@@ -913,8 +947,14 @@ process_debug_str(debug_str)
 	    else if(debug > 2)
 	      ps_global->debug_imap = 1;
 
+	    if(debug > 7)
+	      ps_global->debug_timestamp = 1;
+
 	    if(debug > 8)
 	      ps_global->debug_flush = 1;
+
+	    if(debug > 7)
+	      ps_global->debug_tcp = 1;
 	}
     }
 
