@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: context.c,v 4.59 1999/08/04 00:35:00 mikes Exp $";
+static char rcsid[] = "$Id: context.c,v 4.65 2000/11/09 22:26:47 hubert Exp $";
 #endif
 /*
  * Program:	Mailbox Context Management
@@ -18,7 +18,7 @@ static char rcsid[] = "$Id: context.c,v 4.59 1999/08/04 00:35:00 mikes Exp $";
  * permission of the University of Washington.
  *
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-1998 by the University of Washington.
+ * 1989-2000 by the University of Washington.
  *
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -49,10 +49,12 @@ static char rcsid[] = "$Id: context.c,v 4.59 1999/08/04 00:35:00 mikes Exp $";
  *
  */
 char *
-context_digest (context, scontext, host, rcontext, view)
+context_digest (context, scontext, host, rcontext, view, len)
     char *context, *scontext, *host, *rcontext, *view;
+    size_t len;
 {
     char *p, *viewp = view, tmp[MAILTMPLEN];
+    char *scontextp = scontext;
     int   i = 0;
 
     if((p = context) == NULL || *p == '\0'){
@@ -68,7 +70,7 @@ context_digest (context, scontext, host, rcontext, view)
     /* find hostname if requested and exists */
     if(*p == '{' || (*p == '*' && *++p == '{')){
 	for(p++; *p && *p != '}' ; p++)
-	  if(host)
+	  if(host && p-context < len-1)
 	    *host++ = *p;		/* copy host if requested */
 
 	while(*p && *p != '}')		/* find end of imap host */
@@ -81,8 +83,10 @@ context_digest (context, scontext, host, rcontext, view)
     }
 
     for(; *p ; p++){			/* get thru context */
-	if(rcontext)
-	  rcontext[i++] = *p;		/* copy if requested */
+	if(rcontext && i < len-1)
+	  rcontext[i] = *p;		/* copy if requested */
+	
+	i++;
 
 	if(*p == '['){			/* done? */
 	    if(*++p == '\0')		/* look ahead */
@@ -104,7 +108,7 @@ context_digest (context, scontext, host, rcontext, view)
 	      break;
 	}
 
-	if(viewp)
+	if(viewp && viewp-view < len-1)
 	  *viewp++ = *p;
     }
 
@@ -112,43 +116,55 @@ context_digest (context, scontext, host, rcontext, view)
       return("No ']' in context");
 
     for(; *p ; p++){			/* trailing context ? */
-	if(rcontext)
-	  rcontext[i++] = *p;
+	if(rcontext && i < len-1)
+	  rcontext[i] = *p;
+	i++;
     }
 
     if(host) *host = '\0';
-    if(rcontext) rcontext[i] = '\0';
+    if(rcontext && i < len) rcontext[i] = '\0';
     if(viewp) {
 /* MAIL_LIST: dealt with it in new_context since could be either '%' or '*'
-	if(viewp == view)
+	if(viewp == view && viewp-view < len-1)
 	  *viewp++ = '*';
 */
 
 	*viewp = '\0';
     }
 
-    if(scontext){			/* sprint'able context request ? */
-	if(*context == '*')
-	  *scontext++ = *context++;
+    if(scontextp){			/* sprint'able context request ? */
+	if(*context == '*'){
+	    if(scontextp-scontext < len-1)
+	      *scontextp++ = *context;
+
+	    context++;
+	}
 
 	if(*context == '{'){
-	    while(*context && *context != '}')
-	      *scontext++ = *context++;
+	    while(*context && *context != '}'){
+		if(scontextp-scontext < len-1)
+		  *scontextp++ = *context;
+		
+		context++;
+	    }
 
-	    *scontext++ = '}';
+	    *scontextp++ = '}';
 	}
 
 	for(p = rcontext; *p ; p++){
 	    if(*p == '[' && *(p+1) == ']'){
-		*scontext++ = '%';	/* replace "[]" with "%s" */
-		*scontext++ = 's';
+		if(scontextp-scontext < len-2){
+		    *scontextp++ = '%';	/* replace "[]" with "%s" */
+		    *scontextp++ = 's';
+		}
+
 		p++;			/* skip ']' */
 	    }
-	    else
-	      *scontext++ = *p;
+	    else if(scontextp-scontext < len-1)
+	      *scontextp++ = *p;
 	}
 
-	*scontext = '\0';
+	*scontextp = '\0';
     }
 
     return(NULL);			/* no problems to report... */
@@ -161,27 +177,46 @@ context_digest (context, scontext, host, rcontext, view)
  *          No context applied if error
  */
 char *
-context_apply(b, c, n)
+context_apply(b, c, name, len)
     char      *b;
     CONTEXT_S *c;
-    char      *n;
+    char      *name;
+    size_t     len;
 {
-    if(!c || IS_REMOTE(n) || (!IS_REMOTE(c->context) && is_absolute_path(n))){
-	strcpy(b, n);				/* no context! */
+    if(!c || IS_REMOTE(name) ||
+       (!IS_REMOTE(c->context) && is_absolute_path(name))){
+	strncpy(b, name, len-1);			/* no context! */
     }
-    else if(n[0] == '#'){
+    else if(name[0] == '#'){
 	if(IS_REMOTE(c->context)){
-	    char *p = strchr(c->context, '}');	/* n specifies namespace */
-	    sprintf(b, "%.*s%s", p - c->context + 1, c->context, n);
+	    char *p = strchr(c->context, '}');	/* name specifies namespace */
+	    sprintf(b, "%.*s", min(p - c->context + 1, len-1), c->context);
+	    b[min(p - c->context + 1, len-1)] = '\0';
+	    sprintf(b+strlen(b), "%.*s", len-1-strlen(b), name);
+	}
+	else{
+	    strncpy(b, name, len-1);
 	}
-	else
-	  strcpy(b, n);
     }
-    else if(c->dir->ref)			/* has reference string! */
-      sprintf(b, "%s%s", c->dir->ref, n);
-    else					/* no ref, apply to context */
-      sprintf(b, c->context, n);
+    else if(c->dir && c->dir->ref){		/* has reference string! */
+	sprintf(b, "%.*s", len-1, c->dir->ref);
+	b[len-1] = '\0';
+	sprintf(b+strlen(b), "%.*s", len-1-strlen(b), name);
+    }
+    else{					/* no ref, apply to context */
+	if(strlen(c->context) + strlen(name) < len)
+	  sprintf(b, c->context, name);
+	else{
+	    char *t;
 
+	    t = (char *)fs_get((strlen(c->context)+strlen(name))*sizeof(char));
+	    sprintf(t, c->context, name);
+	    strncpy(b, t, len-1);
+	    fs_give((void **)&t);
+	}
+    }
+
+    b[len-1] = '\0';
     return(b);
 }
 
@@ -213,8 +248,8 @@ context_create (context, stream, mailbox)
 {
   char tmp[MAILTMPLEN];		/* must be within context */
 
-  return(cntxt_allowed(context_apply(tmp, context, mailbox))
-	 ? mail_create(stream,tmp) : 0L);
+  return(cntxt_allowed(context_apply(tmp, context, mailbox, sizeof(tmp)))
+	 ? pine_mail_create(stream,tmp) : 0L);
 }
 
 
@@ -235,7 +270,7 @@ context_open (context, old, name, opt)
 {
   char tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
 
-  return(cntxt_allowed(context_apply(tmp, context, name))
+  return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
 	  ? pine_mail_open(old,tmp,opt) : (MAILSTREAM *)NULL);
 }
 
@@ -257,7 +292,7 @@ context_status (context, stream, name, opt)
 {
     char tmp[MAILTMPLEN];	/* build FQN from ambiguous name */
 
-    return(cntxt_allowed(context_apply(tmp, context, name))
+    return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
 	     ? mail_status(stream,tmp,opt) : 0L);
 }
 
@@ -278,8 +313,8 @@ context_rename (context, stream, old, new)
 {
     char tmp[MAILTMPLEN],tmp2[MAILTMPLEN];
 
-    return((cntxt_allowed(context_apply(tmp, context, old))
-	    && cntxt_allowed(context_apply(tmp2, context, new)))
+    return((cntxt_allowed(context_apply(tmp, context, old, sizeof(tmp)))
+	    && cntxt_allowed(context_apply(tmp2, context, new, sizeof(tmp2))))
 	     ? mail_rename(stream,tmp,tmp2) : 0L);
 }
 
@@ -299,7 +334,7 @@ context_delete (context, stream, name)
 {
     char tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
 
-    return(cntxt_allowed(context_apply(tmp, context, name))
+    return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
 	     ? mail_delete(stream, tmp) : 0L);
 }
 
@@ -321,7 +356,7 @@ context_append (context, stream, name, msg)
 {
     char tmp[MAILTMPLEN];	/* build FQN from ambiguous name */
 
-    return(cntxt_allowed(context_apply(tmp, context, name))
+    return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
 	     ? mail_append(stream, tmp, msg) : 0L);
 }
 
@@ -345,11 +380,35 @@ context_append_full (context, stream, name, flags, date, msg)
 {
     char tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
 
-    return(cntxt_allowed(context_apply(tmp, context, name))
+    return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
 	     ? mail_append_full(stream, tmp, flags, date, msg) : 0L);
 }
 
 
+/* Context Manager append multiple message
+ * Accepts: context
+ *	    mail stream
+ *	    destination mailbox
+ *	    append data callback
+ *	    arbitrary ata for callback use
+ * Returns: T on success, NIL on failure
+ */
+
+long
+context_append_multiple (context, stream, name, af, data)
+    CONTEXT_S  *context;
+    MAILSTREAM *stream;
+    char       *name;
+    append_t   af;
+    void       *data;
+{
+    char tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
+
+    return(cntxt_allowed(context_apply(tmp, context, name, sizeof(tmp)))
+	     ? mail_append_multiple(stream, tmp, af, data) : 0L);
+}
+
+
 /* Mail copy message(s)
  * Accepts: context
  *	    mail stream
@@ -366,7 +425,7 @@ context_copy (context, stream, sequence, name)
 {
     char *s, tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
 
-    if(context_apply(tmp, context, name)[0] == '{'){
+    if(context_apply(tmp, context, name, sizeof(tmp))[0] == '{'){
 	if(s = strindex(tmp, '}'))
 	  s++;
 	else
@@ -399,72 +458,5 @@ context_same_stream(context, name, stream)
     extern MAILSTREAM *same_stream();
     char tmp[MAILTMPLEN];		/* build FQN from ambiguous name */
 
-    return(same_stream(context_apply(tmp, context, name), stream));
+    return(same_stream(context_apply(tmp, context, name, sizeof(tmp)), stream));
 }
-
-
-#ifdef NEWBB
-
-/*  This is code for inclusion in the c-client to support the command
-    to discover all the new newly created news groups, since the last
-    time the subscription screen was viewed. 
-
-    Some of this code goes un nntpunx.c and one line in nntpunx.h
-
-    It's not clear whether or not this code will be included as part
-    of the official c-client distribution so it's included here.
-*/
-
-#ifdef INCLUDE_THIS_CODE_AT_THE_END_OF_NNTPUNX_C
-
-/* NNTP mail find list of all bboards
- * Accepts: mail stream
- *	    pattern to search
- *          last time that new groups were checked for in NNTP format
- */
-
-void nntp_find_new_bboard (stream, pat, lasttime)
-    MAILSTREAM *stream;
-    char       *pat, *lasttime;
-{
-  char *s,*t,*bbd,*patx,tmp[MAILTMPLEN], checktime[20];
-				/* use .newsrc if a stream given */
-  if (stream && LOCAL && LOCAL->nntpstream) {
-				/* begin with a host specification? */
-    if (((*pat == '{') || ((*pat == '*') && (pat[1] == '{'))) &&
-	(t = strchr (pat,'}')) && *(patx = ++t)) {
-      if (*pat == '*') pat++;	/* yes, skip leading * (old Pine behavior) */
-      strcpy (tmp,pat);		/* copy host name */
-      bbd = tmp + (patx - pat);	/* where we write the bboards */
-    }
-    else {			/* no host specification */
-      bbd = tmp;		/* no prefix */
-      patx = pat;		/* use entire specification */
-    }
-				/* ask server for all active newsgroups */
-    if (!(smtp_send (LOCAL->nntpstream,"NEWGROUPS",lasttime) == NNTPNEWLIST))
-      return;
-				/* process data until we see final dot */
-    while ((s = tcp_getline (LOCAL->nntpstream->tcpstream)) && *s != '.') {
-				/* tie off after newsgroup name */
-      if (t = strchr (s,' ')) *t = '\0';
-      if (pmatch (s,patx)) {	/* report to main program if have match */
-	strcpy (bbd,s);		/* write newsgroup name after prefix */
-	mm_bboard(tmp);
-      }
-      fs_give ((void **) &s);	/* clean up */
-    }
-  }
-}
-
-     
-#endif
-
-
-#ifdef INCLUDE_THIS_CODE_NEAR_THE_TOP_OF_NNTPUNX_H
-
-#define NNTPNEWLIST (long) 231  /* NNTP list of new news groups */
-
-#endif
-
-#endif /* NEWBB */
