@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c,v 4.119 2000/11/11 00:03:35 hubert Exp $";
+static char rcsid[] = "$Id: help.c,v 4.124 2001/08/07 22:49:47 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: help.c,v 4.119 2000/11/11 00:03:35 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2000 by the University of Washington.
+   1989-2001 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -71,26 +71,6 @@ INST_KEY_MENU(help_keymenu, help_keys);
 #define	HLP_ALL_KEY	9
 
 
-/*
- * Keys for the Report Bug screen.
- */
-static struct key gripe_modal_keys[] = 
-       {NULL_MENU,
-	NULL_MENU,
-	{"Ret","Finished",{MC_EXIT,2,{ctrl('m'),ctrl('j')}},KS_NONE},
-	NULL_MENU,
-	NULL_MENU,
-	NULL_MENU,
-	PREVPAGE_MENU,
-	NEXTPAGE_MENU,
-	NULL_MENU,
-	NULL_MENU,
-	NULL_MENU,
-	NULL_MENU
-       };
-INST_KEY_MENU(gripe_modal_km, gripe_modal_keys);
-
-
 typedef struct _help_scroll {
     unsigned	keys_formatted:1;	/* Has full keymenu been formatted? */
     char      **help_source;		/* Source of displayed help text */
@@ -139,11 +119,12 @@ static char att_cur_msg[] = "\
 /*
  * Internal prototypes
  */
-int	 helper_internal PROTO((HelpType, char *, char *, int));
+int	 helper_internal PROTO((HelpType, char **, char *, char *, int));
 HelpType help_name2section PROTO((char *, int));
 int	 help_processor PROTO((int, MSGNO_S *, SCROLL_S *));
 void	 help_keymenu_tweek PROTO((SCROLL_S *, int));
 int	 help_bogus_input PROTO((int));
+char   **get_supported_options PROTO((void));
 void     print_all_help PROTO((void));
 void	 print_help_page_title PROTO((char *, HPRT_S *));
 int	 print_help_page_break PROTO((long, char *, LT_INS_S **, void *));
@@ -164,6 +145,9 @@ int	 help_subsection_popup PROTO((SCROLL_S *, int));
 
     Args: text   -- The help text to display (from pine.help --> helptext.c)
           title  -- The title of the help text 
+
+	  The dumb otext argument is for when we want to pass in a char **
+	  data item no matter what the type of HelpType is.
   
   Result: format text and call scroller
 
@@ -173,8 +157,9 @@ The list is ended with a page line number of -1. Line number 0 is also
 the first line in the text.
   -----*/
 int
-helper_internal(text, frag, title, flags)
+helper_internal(text, otext, frag, title, flags)
     HelpType  text;
+    char    **otext;
     char     *frag;
     char     *title;
     int	      flags;
@@ -188,16 +173,21 @@ helper_internal(text, frag, title, flags)
     HELP_SCROLL_S   hscroll;
     gf_io_t	    pc;
 #ifdef	HELPFILE
-    char	  **dynamic_text;
+    char	  **dynamic_text = NULL;
 #endif /* HELPFILE */
 
     dprint(1, (debugfile, "\n\n    ---- HELPER ----\n"));
 
 #ifdef	HELPFILE
-    if((shown_text = dynamic_text = get_help_text(text)) == NULL)
+    if(otext)
+      shown_text = otext;
+    else if((shown_text = dynamic_text = get_help_text(text)) == NULL)
       return(cmd);
 #else
-    shown_text = text;
+    if(otext)
+      shown_text = otext;
+    else
+      shown_text = text;
 #endif /* HELPFILE */
 
     if(F_ON(F_BLANK_KEYMENU,ps_global)){
@@ -229,7 +219,7 @@ helper_internal(text, frag, title, flags)
 	      gf_link_filter(gf_html2plain,
 			     gf_html2plain_opt("x-pine-help:",
 					ps_global->ttyo->screen_cols,
-					GFHP_HANDLES | GFHP_LOCAL_HANDLES));
+					&handles, GFHP_LOCAL_HANDLES));
 	    else
 	      gf_link_filter(gf_wrap, gf_wrap_filter_opt(
 						  ps_global->ttyo->screen_cols,
@@ -398,7 +388,8 @@ helper_internal(text, frag, title, flags)
     while(cmd == MC_RESIZE);
 
 #ifdef	HELPFILE
-    free_list_array(&dynamic_text);
+    if(dynamic_text)
+      free_list_array(&dynamic_text);
 #endif
 
     return(cmd);
@@ -414,7 +405,7 @@ helper(text, title, flags)
     char     *title;
     int	      flags;
 {
-    return(helper_internal(text, NULL, title, flags));
+    return(helper_internal(text, NULL, NULL, title, flags));
 }
 
 
@@ -535,7 +526,8 @@ print_help(text)
 	int   i;
 	char *p;
 
-	gf_link_filter(gf_html2plain,gf_html2plain_opt(NULL,80,GFHP_STRIPPED));
+	gf_link_filter(gf_html2plain,
+		       gf_html2plain_opt(NULL,80,NULL,GFHP_STRIPPED));
 	for(i = 1; i <= 5 && text[i]; i++)
 	  if(!struncmp(text[i], "<title>", 7)
 	     && (p = srchstr(text[i] + 7, "</title>"))
@@ -689,7 +681,7 @@ url_local_helper(url)
 	if(newhelp != NO_HELP){
 	    int rv;
 
-	    rv = helper_internal(newhelp, frag, "HELP SUB-SECTION",
+	    rv = helper_internal(newhelp, NULL, frag, "HELP SUB-SECTION",
 				 HLPD_NEWWIN | HLPD_SIMPLE | HLPD_FROMHELP);
 	    ps_global->mangled_screen = 1;
 	    return((rv == MC_EXIT) ? 2 : 1);
@@ -702,6 +694,133 @@ url_local_helper(url)
 }
 
 
+char **
+get_supported_options()
+{
+    char         **config;
+    DRIVER        *d;
+    AUTHENTICATOR *a;
+    char          *title = "Supported features in this Pine";
+    char           sbuf[MAX_SCREEN_COLS+1];
+    int            cnt, len, cols, disabled, any_disabled = 0;;
+
+    /*
+     * Line count:
+     *   Title + blank			= 2
+     *   SSL Title + SSL line + blank	= 3
+     *   Auth title + blank		= 2
+     *   Driver title + blank		= 2
+     *   LDAP title + LDAP line 	= 2
+     *   Disabled explanation + blank line = 4
+     *   end				= 1
+     */
+    cnt = 16;
+    for(a = mail_lookup_auth(1); a; a = a->next)
+      cnt++;
+    for(d = (DRIVER *)mail_parameters(NIL, GET_DRIVERS, NIL);
+	d; d = d->next)
+      cnt++;
+
+    config = (char **)fs_get(cnt * sizeof(char *));
+    memset(config, 0, cnt * sizeof(char *));
+
+    cols = ps_global->ttyo ? ps_global->ttyo->screen_cols : 0;
+    len = strlen(title);
+    sprintf(sbuf, "%*s%s", cols > len ? (cols-len)/2 : 0, "", title);
+
+    config[cnt=0] = cpystr(sbuf);
+    config[++cnt] = cpystr("");
+
+    config[++cnt] = cpystr("Encryption:");
+    if(mail_parameters(NIL, GET_SSLDRIVER, NIL))
+      config[++cnt] = cpystr("  TLS and SSL");
+    else
+      config[++cnt] = cpystr("  None (no TLS or SSL)");
+
+    config[++cnt] = cpystr("");
+    config[++cnt] = cpystr("Authenticators:");
+    for(a = mail_lookup_auth(1); a; a = a->next){
+	disabled = (a->client == NULL && a->server == NULL);
+	any_disabled += disabled;
+	sprintf(sbuf, "  %s%s", a->name, disabled ? " (disabled)" : "");
+	config[++cnt] = cpystr(sbuf);
+    }
+
+    config[++cnt] = cpystr("");
+    config[++cnt] = cpystr("Mailbox drivers:");
+    for(d = (DRIVER *)mail_parameters(NIL, GET_DRIVERS, NIL);
+	d; d = d->next){
+	disabled = (d->flags & DR_DISABLE);
+	any_disabled += disabled;
+	sprintf(sbuf, "  %s%s", d->name, disabled ? " (disabled)" : "");
+	config[++cnt] = cpystr(sbuf);
+    }
+
+    config[++cnt] = cpystr("");
+    config[++cnt] = cpystr("Directories:");
+#ifdef	ENABLE_LDAP
+      config[++cnt] = cpystr("  LDAP");
+#else
+      config[++cnt] = cpystr("  None (no LDAP)");
+#endif
+
+    if(any_disabled){
+	config[++cnt] = cpystr("");
+	if(ps_global->ttyo)
+	  config[++cnt] = cpystr("Authenticators may be disabled because of the \"disable-these-authenticators\" hidden config option. Mailbox drivers may be disabled because of the \"disable-these-drivers\" hidden config option.");
+	else{
+	    config[++cnt] = cpystr("Authenticators may be disabled because of the \"disable-these-authenticators\"");
+	    config[++cnt] = cpystr("hidden config option. Mailbox drivers may be disabled because of the");
+	    config[++cnt] = cpystr("\"disable-these-drivers\" hidden config option.");
+	}
+    }
+
+    config[++cnt] = NULL;
+
+    return(config);
+}
+
+
+int
+url_local_config(url)
+    char *url;
+{
+    if(!struncmp(url, "x-pine-config:", 14)){
+	char **config;
+	int    rv;
+
+	config = get_supported_options();
+	if(config){
+	    rv = helper_internal(NO_HELP, config, NULL, "HELP CONFIG",
+				 HLPD_NEWWIN | HLPD_SIMPLE | HLPD_FROMHELP);
+	    free_list_array(&config);
+	}
+
+	ps_global->mangled_screen = 1;
+	return((rv == MC_EXIT) ? 2 : 1);
+    }
+
+    q_status_message1(SM_ORDER | SM_DING, 0, 3,
+		      "Unrecognized Internal help: \"%s\"", url);
+    return(0);
+}
+
+
+void
+dump_supported_options()
+{
+    char **config;
+    char **p;
+    FILE  *f = stdout;
+
+    config = get_supported_options();
+    if(config){
+	display_args_err(NULL, config, 0);
+	free_list_array(&config);
+    }
+}
+
+
 HelpType
 help_name2section(url, url_len)
     char *url;
@@ -878,7 +997,7 @@ review_messages(title)
     char           *p, *q, *last_space, *cp;
     char            save, buf[MAX_SCREEN_COLS + 1];
     register char **e;
-    register int    i, j, k, m, line_len;
+    register int    i, k, m, line_len;
     SCROLL_S	    sargs;
 
     e = stat_msgs;
@@ -897,7 +1016,7 @@ review_messages(title)
     *cur = '\0';
 
     /* allocate strings */
-    for(k = 0, j = 0, i = latest; k < NSTATUS; k++){
+    for(k = 0, i = latest; k < NSTATUS; k++){
 	i = (i + 1) % NSTATUS;
 	if(e[i] && *e[i]){
 	    p = q = e[i];
@@ -1364,7 +1483,6 @@ ADDRESS *
 gripe_token_addr(token)
     char *token;
 {
-    int	     rv = 0;
     char    *p;
     ADDRESS *a = NULL;
 
@@ -1379,7 +1497,6 @@ gripe_token_addr(token)
 				  && ps_global->VAR_LOCAL_FULLNAME[0])
 				    ? ps_global->VAR_LOCAL_FULLNAME 
 				    : "Place to report Pine Bugs");
-	    rv = 1;
 	}
 	else if(!strcmp(token, "BUGS_ADDRESS_")){
 	    p = (ps_global->VAR_BUGS_ADDRESS
@@ -1390,7 +1507,6 @@ gripe_token_addr(token)
 				  && ps_global->VAR_BUGS_FULLNAME[0])
 				    ? ps_global->VAR_BUGS_FULLNAME 
 				    : "Place to report Pine Bugs");
-	    rv = 1;
 	}
     }
 
