@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.190 2001/08/17 18:10:21 hubert Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.204 2001/11/28 19:12:44 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -96,12 +96,14 @@ char *ps_get PROTO((size_t));
 #ifdef	PASSFILE
 char  xlate_in PROTO((int));
 char  xlate_out PROTO((char));
-char *passfile_name PROTO((char *, char *));
+char *passfile_name PROTO((char *, char *, size_t));
 int   read_passfile PROTO((char *, MMLOGIN_S **));
 void  write_passfile PROTO((char *, MMLOGIN_S *));
 int   get_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
+int   is_using_passfile PROTO(());
 void  set_passfile_passwd PROTO((char *, char *, char *, STRLIST_S *, int));
 char *get_passfile_user PROTO((char *, STRLIST_S *));
+void  update_passfile_hostlist PROTO((char *, char *, STRLIST_S *, int));
 #endif
 
 
@@ -190,7 +192,9 @@ mm_log(string, errflg)
 		  ? "warn"
 		  : (errflg == PARSE)
 		      ? "parse"
-		      : "babble",
+		      : (errflg == TCPDEBUG)
+			  ? "tcp"
+			  : "babble",
 	    string));
 
     if(errflg == ERROR && !strncmp(string, "[TRYCREATE]", 11)){
@@ -582,32 +586,81 @@ mm_login(mb, user, pwd, trial)
     char   *pwd;
     long    trial;
 {
-    char      prompt[1000], *last;
+    char      prompt[1000], *last, *logleadin;
+    char      port[20], non_def_port[20], insecure[20];
     char      defuser[NETMAXUSER];
+    char      hostleadin[20], hostname[200], defubuf[200];
+    char      pwleadin[50];
+    char      hostlist0[MAILTMPLEN], hostlist1[MAILTMPLEN];
+    char     *insec = " (INSECURE)";
     STRLIST_S hostlist[2];
     HelpType  help ;
-    int       i, j, goal, ugoal, len, lastlen, dlen, rc, q_line, flags;
+    int       len, rc, q_line, flags;
+    int       oespace, avail, need;
+    struct servent *sv;
 #define NETMAXPASSWD 100
 
-    dprint(9, (debugfile, "mm_login trial=%ld mb->user=%s\n", trial,
-	       mb->user ? mb->user : "(null)"));
+    dprint(9, (debugfile, "mm_login trial=%ld user=%s service=%s%s%s\n",
+	       trial, mb->user ? mb->user : "(null)",
+	       mb->service ? mb->service : "(null)",
+	       mb->port ? " port=" : "",
+	       mb->port ? comatose(mb->port) : ""));
     q_line = -(ps_global->ttyo ? ps_global->ttyo->footer_rows : 3);
 
     /* make sure errors are seen */
     if(ps_global->ttyo)
       flush_status_messages(0);
 
+    /*
+     * Add port number to hostname if going through a tunnel or something
+     */
+    non_def_port[0] = '\0';
+    if(mb->port && mb->service &&
+       (sv = getservbyname(mb->service, "tcp")) &&
+       (mb->port != ntohs(sv->s_port))){
+	sprintf(non_def_port, ":%lu", mb->port);
+	dprint(9, (debugfile, "mm_login: using non-default port=%s\n",
+		   non_def_port));
+    }
+
     /*
      * set up host list for sybil servers...
      */
-    hostlist[0].name = mb->host;
-    if(mb->orighost[0] && strucmp(mb->host, mb->orighost)){
-	hostlist[0].next = &hostlist[1];
-	hostlist[1].name = mb->orighost;
-	hostlist[1].next = NULL;
+    if(*non_def_port){
+	strncpy(hostlist0, mb->host, sizeof(hostlist0)-1);
+	hostlist0[sizeof(hostlist0)-1] = '\0';
+	strncat(hostlist0, non_def_port, sizeof(hostlist0)-strlen(hostlist0)-1);
+	hostlist0[sizeof(hostlist0)-1] = '\0';
+	hostlist[0].name = hostlist0;
+	if(mb->orighost && mb->orighost[0] && strucmp(mb->host, mb->orighost)){
+	    strncpy(hostlist1, mb->orighost, sizeof(hostlist1)-1);
+	    hostlist1[sizeof(hostlist1)-1] = '\0';
+	    strncat(hostlist1, non_def_port, sizeof(hostlist1)-strlen(hostlist1)-1);
+	    hostlist1[sizeof(hostlist1)-1] = '\0';
+	    hostlist[0].next = &hostlist[1];
+	    hostlist[1].name = hostlist1;
+	    hostlist[1].next = NULL;
+	}
+	else
+	  hostlist[0].next = NULL;
+    }
+    else{
+	hostlist[0].name = mb->host;
+	if(mb->orighost && mb->orighost[0] && strucmp(mb->host, mb->orighost)){
+	    hostlist[0].next = &hostlist[1];
+	    hostlist[1].name = mb->orighost;
+	    hostlist[1].next = NULL;
+	}
+	else
+	  hostlist[0].next = NULL;
+    }
+
+    if(hostlist[0].name){
+	dprint(9, (debugfile, "mm_login: host=%s\n", hostlist[0].name));
+	if(hostlist[0].next && hostlist[1].name){
+	    dprint(9, (debugfile, "mm_login: orighost=%s\n", hostlist[1].name));
+	}
     }
-    else
-      hostlist[0].next = NULL;
 
     /*
      * Initialize user name with either 
@@ -631,10 +684,12 @@ mm_login(mb, user, pwd, trial)
 
 #ifdef	PASSFILE
 	/* check to see if there's a password left over from last session */
-	if(get_passfile_passwd(ps_global->pinerc,pwd,
-			       user,&hostlist[0],(mb->sslflag||mb->tlsflag))){
+	if(get_passfile_passwd(ps_global->pinerc, pwd,
+			       user, hostlist, (mb->sslflag||mb->tlsflag))){
 	    imap_set_passwd(&mm_login_list, pwd, user,
-			    &hostlist[0], (mb->sslflag||mb->tlsflag), 0, 0);
+			    hostlist, (mb->sslflag||mb->tlsflag), 0, 0);
+	    update_passfile_hostlist(ps_global->pinerc, user, hostlist,
+				     (mb->sslflag||mb->tlsflag));
 	    return;
 	}
 #endif
@@ -673,9 +728,6 @@ mm_login(mb, user, pwd, trial)
      */
     ps_global->mangled_footer = 1;
     if(!*mb->user){
-	char *strtwo, *strthree;
-	char  onebuf[200], twobuf[200], fourbuf[200];
-	int one, two, three, four, five, avail, need;
 
 	help = NO_HELP;
 
@@ -691,61 +743,96 @@ mm_login(mb, user, pwd, trial)
 	/*
 	 * Need space for "+ HOST: "
 	 *                hostname
+	 *                " (INSECURE)"
 	 *                ENTER LOGIN NAME
 	 *                " [defuser] : "
 	 *                about 15 chars for input
 	 */
 
-	sprintf(onebuf, "%sHOST: ",
+	sprintf(hostleadin, "%sHOST: ",
 		(!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
 
-	one = strlen(onebuf);
-	strtwo = mb->host;
-	two = strlen(strtwo);
-	strthree = "  ENTER LOGIN NAME";
-	three = strlen(strthree);
-	sprintf(fourbuf, "%s%.100s%s : ", (*defuser) ? " [" : "",
+	strncpy(hostname, mb->host, sizeof(hostname)-1);
+	hostname[sizeof(hostname)-1] = '\0';
+
+	/*
+	 * Add port number to hostname if going through a tunnel or something
+	 */
+	if(*non_def_port)
+	  strncpy(port, non_def_port, sizeof(port));
+	else
+	  port[0] = '\0';
+	
+	insecure[0] = '\0';
+	/* if not encrypted and SSL/TLS is supported */
+	if(!(mb->sslflag||mb->tlsflag) &&
+	   mail_parameters(NIL, GET_SSLDRIVER, NIL))
+	  strncpy(insecure, insec, sizeof(insecure));
+
+	logleadin = "  ENTER LOGIN NAME";
+
+	sprintf(defubuf, "%s%.100s%s : ", (*defuser) ? " [" : "",
 					  (*defuser) ? defuser : "",
 					  (*defuser) ? "]" : "");
-	four = strlen(fourbuf);
-	five = max(min(15,
-		       (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5),
-		   6);
+	/* space reserved after prompt */
+	oespace = max(min(15, (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5), 6);
+
 	avail = ps_global->ttyo ? ps_global->ttyo->screen_cols : 80;
-	need = one+two+three+four+five;
+	need = strlen(hostleadin) + strlen(hostname) + strlen(port) +
+	       strlen(insecure) + strlen(logleadin) + strlen(defubuf) + oespace;
 
 	if(avail < need){
-	    /* reduce space three */
-	    strthree = " LOGIN";
-	    need -= (three - strlen(strthree));
+	    /* reduce length of logleadin */
+	    len = strlen(logleadin);
+	    logleadin = " LOGIN";
+	    need -= (len - strlen(logleadin));
 
 	    if(avail < need){
-		/* get two or three more spaces here */
-		sprintf(onebuf, "%sHST:",
+		/* get two or three spaces from hostleadin */
+		len = strlen(hostleadin);
+		sprintf(hostleadin, "%sHST:",
 		  (!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+" : "");
-		need -= (one - strlen(onebuf));
+		need -= (len - strlen(hostleadin));
+
+		/* get rid of port */
+		if(avail < need && strlen(port) > 0){
+		    need -= strlen(port);
+		    port[0] = '\0';
+		}
 
 		if(avail < need){
 		    int reduce_to;
+
 		    /*
 		     * Reduce space for hostname. Best we can do is 6 chars
-		     * with host...
+		     * with hos...
 		     */
-		    reduce_to = (need - avail < two - 6) ? (two-(need-avail))
-							 : 6;
-		    strncpy(twobuf, strtwo, reduce_to-3);
-		    strncpy(twobuf+reduce_to-3, "...", 4);
-		    need -= (two - strlen(twobuf));
-		    strtwo = twobuf;
+		    reduce_to = (need - avail < strlen(hostname) - 6) ? (strlen(hostname)-(need-avail)) : 6;
+		    len = strlen(hostname);
+		    strncpy(hostname+reduce_to-3, "...", 4);
+		    need -= (len - strlen(hostname));
+
+		    if(avail < need && strlen(insecure) > 0){
+			if(need - avail <= 3){
+			    need -= 3;
+			    insecure[strlen(insecure)-4] = ')';
+			    insecure[strlen(insecure)-3] = '\0';
+			}
+			else{
+			    need -= strlen(insecure);
+			    insecure[0] = '\0';
+			}
+		    }
 
 		    if(avail < need){
-			if(four > 3){
-			    strncpy(fourbuf, " [..] :", 9);
-			    need -= (four - strlen(fourbuf));
+			if(strlen(defubuf) > 3){
+			    len = strlen(defubuf);
+			    strncpy(defubuf, " [..] :", 9);
+			    need -= (len - strlen(defubuf));
 			}
 
 			if(avail < need)
-			  strncpy(fourbuf, ":", 2);
+			  strncpy(defubuf, ":", 2);
 
 			/*
 			 * If it still doesn't fit, optionally_enter gets
@@ -756,14 +843,28 @@ mm_login(mb, user, pwd, trial)
 	    }
 	}
 
-	sprintf(prompt, "%.200s%.200s%.200s%.200s",
-		onebuf, strtwo, strthree, fourbuf);
+	sprintf(prompt, "%.20s%.200s%.20s%.20s%.50s%.200s",
+		hostleadin, hostname, port, insecure, logleadin, defubuf);
 
 	while(1) {
 	    if(ps_global->ttyo)
 	      mm_login_alt_cue(mb);
 
 	    flags = OE_APPEND_CURRENT;
+#ifdef _WINDOWS
+	    if(!ps_global->ttyo){
+		if(!*user && *defuser)
+		  strcpy(user, defuser);
+		rc = os_login_dialog(mb, user, NETMAXUSER, pwd, NETMAXPASSWD,
+#ifdef PASSFILE
+				is_using_passfile() ? 1 :
+#endif /* PASSFILE */
+				0, 0);
+		if(rc == 0 && *user && *pwd)
+		  goto nopwpmt;
+	    }
+	    else
+#endif /* _WINDOWS */
 	    rc = optionally_enter(user, q_line, 0, NETMAXUSER,
 				  prompt, NULL, help, &flags);
 	    if(rc == 3) {
@@ -804,10 +905,10 @@ mm_login(mb, user, pwd, trial)
 	  return;
 
 #ifdef	PASSFILE
-	if(get_passfile_passwd(ps_global->pinerc,pwd,
-			       user,&hostlist[0],(mb->sslflag||mb->tlsflag))){
+	if(get_passfile_passwd(ps_global->pinerc, pwd,
+			       user, hostlist, (mb->sslflag||mb->tlsflag))){
 	    imap_set_passwd(&mm_login_list, pwd, user,
-			    &hostlist[0], (mb->sslflag||mb->tlsflag), 0, 0);
+			    hostlist, (mb->sslflag||mb->tlsflag), 0, 0);
 	    return;
 	}
 #endif
@@ -818,59 +919,121 @@ mm_login(mb, user, pwd, trial)
      */
     help = NO_HELP;
 
-    /* Dress up long host/user names */
-    /* leave space for "HOST", "USER" "ENTER PWD", 12 for user 6 for pwd */
-    sprintf(prompt, "%sHOST: ",
+    /*
+     * Need space for "+ HOST: "
+     *                hostname
+     *                " (INSECURE) "
+     *                "  USER: "
+     *                user
+     *                "  ENTER PASSWORD: "
+     *                about 15 chars for input
+     */
+    
+    sprintf(hostleadin, "%sHOST: ",
 	    (!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
-    len = strlen(prompt);
-    goal  = strlen(mb->host);
-    ugoal = strlen(user);
-    if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
-		(len + 8 + 18 + 6)) < 14){
-	goal = 0;		/* no host! */
-	if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
-		(6 + 18 + 6)) <= 6){
-	    ugoal = 0;		/* no user! */
-	    if((i = (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80) -
-		(18 + 6)) <= 0)
-	      i = 0;
+
+    strncpy(hostname, mb->host, sizeof(hostname)-1);
+    hostname[sizeof(hostname)-1] = '\0';
+
+    /*
+     * Add port number to hostname if going through a tunnel or something
+     */
+    if(*non_def_port)
+      strncpy(port, non_def_port, sizeof(port));
+    else
+      port[0] = '\0';
+    
+    insecure[0] = '\0';
+    /* if not encrypted and SSL/TLS is supported */
+    if(!(mb->sslflag||mb->tlsflag) &&
+       mail_parameters(NIL, GET_SSLDRIVER, NIL))
+      strncpy(insecure, insec, sizeof(insecure));
+    
+    logleadin = "  USER: ";
+
+    strncpy(defubuf, user, sizeof(defubuf)-1);
+    defubuf[sizeof(defubuf)-1] = '\0';
+
+    strncpy(pwleadin, "  ENTER PASSWORD: ", sizeof(pwleadin)-1);
+    pwleadin[sizeof(pwleadin)-1] = '\0';
+
+    /* space reserved after prompt */
+    oespace = max(min(15, (ps_global->ttyo ? ps_global->ttyo->screen_cols : 80)/5), 6);
+
+    avail = ps_global->ttyo ? ps_global->ttyo->screen_cols : 80;
+    need = strlen(hostleadin) + strlen(hostname) + strlen(port) +
+	   strlen(insecure) + strlen(logleadin) + strlen(defubuf) +
+	   strlen(pwleadin) + oespace;
+    
+    if(avail < need){
+	logleadin = logleadin + 1;
+	need--;
+	rplstr(pwleadin, 1, "");
+	need--;
+
+	if(avail < need){
+	    /* get two or three spaces from hostleadin */
+	    len = strlen(hostleadin);
+	    sprintf(hostleadin, "%sHST:",
+	      (!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+" : "");
+	    need -= (len - strlen(hostleadin));
+
+	    /* get rid of port */
+	    if(avail < need && strlen(port) > 0){
+		need -= strlen(port);
+		port[0] = '\0';
+	    }
+
+	    if(avail < need){
+		len = strlen(pwleadin);
+		strncpy(pwleadin, " PASSWORD: ", sizeof(pwleadin)-1);
+		pwleadin[sizeof(pwleadin)-1] = '\0';
+		need -= (len - strlen(pwleadin));
+	    }
 	}
-	else{
-	    ugoal = i;		/* whatever's left */
-	    i     = 0;
+
+	if(avail < need){
+	    int reduce_to;
+
+	    /*
+	     * Reduce space for hostname. Best we can do is 6 chars
+	     * with hos...
+	     */
+	    reduce_to = (need - avail < strlen(hostname) - 6) ? (strlen(hostname)-(need-avail)) : 6;
+	    len = strlen(hostname);
+	    strncpy(hostname+reduce_to-3, "...", 4);
+	    need -= (len - strlen(hostname));
+	    
+	    if(avail < need && strlen(insecure) > 0){
+		if(need - avail <= 3){
+		    need -= 3;
+		    insecure[strlen(insecure)-4] = ')';
+		    insecure[strlen(insecure)-3] = '\0';
+		}
+		else{
+		    need -= strlen(insecure);
+		    insecure[0] = '\0';
+		}
+	    }
+
+	    if(avail < need){
+		len = strlen(logleadin);
+		logleadin = " ";
+		need -= (len - strlen(logleadin));
+
+		if(avail < need){
+		    reduce_to = (need - avail < strlen(defubuf) - 6) ? (strlen(defubuf)-(need-avail)) : 0;
+		    if(reduce_to)
+		      strncpy(defubuf+reduce_to-3, "...", 4);
+		    else
+		      defubuf[0] = '\0';
+		}
+	    }
 	}
     }
-    else
-      while(goal + ugoal > i)
-	if(goal > ugoal)
-	  goal--;
-	else
-	  ugoal--;
 
-    if(goal){
-	sprintf(prompt, "%sHOST: ",
-		(!ps_global->ttyo && (mb->sslflag||mb->tlsflag)) ? "+ " : "");
-	for(i = len, j = 0;
-	    i < sizeof(prompt) && (prompt[i] = mb->host[j]); i++, j++)
-	  if(i == goal && mb->host[goal+1] && i < sizeof(prompt)){
-	      strncpy(&prompt[i-3], "...", 4);
-	      break;
-	  }
-    }
-    else
-      i = 0;
-
-    if(ugoal){
-	strncpy(&prompt[i], &"  USER: "[i ? 0 : 2], sizeof(prompt)-i);
-	for(i += strlen(&prompt[i]), j = 0;
-	    i < sizeof(prompt) && (prompt[i] = user[j]); i++, j++)
-	  if(j == ugoal && user[ugoal+1] && i < sizeof(prompt)){
-	      strncpy(&prompt[i-3], "...", 4);
-	      break;
-	  }
-    }
-
-    strncpy(&prompt[i], &"  ENTER PASSWORD: "[i ? 0 : 8], sizeof(prompt)-i);
+    sprintf(prompt, "%.20s%.200s%.20s%.20s%.50s%.200s%.50s",
+	    hostleadin, hostname, port, insecure, logleadin, defubuf, pwleadin);
 
     *pwd = '\0';
     while(1) {
@@ -878,6 +1041,11 @@ mm_login(mb, user, pwd, trial)
 	  mm_login_alt_cue(mb);
 
 	flags = OE_PASSWD;
+#ifdef _WINDOWS
+	if(!ps_global->ttyo)
+	  rc = os_login_dialog(mb, user, NETMAXUSER, pwd, NETMAXPASSWD, 0, 1);
+	else
+#endif
         rc = optionally_enter(pwd, q_line, 0, NETMAXPASSWD,
 			      prompt, NULL, help, &flags);
         if(rc == 3) {
@@ -894,6 +1062,7 @@ mm_login(mb, user, pwd, trial)
         return;
     }
 
+ nopwpmt:
     /* remember the password for next time */
     if(F_OFF(F_DISABLE_PASSWORD_CACHING,ps_global))
       imap_set_passwd(&mm_login_list, pwd, user, hostlist,
@@ -1534,8 +1703,8 @@ pine_sslfailure(host, reason, flags)
     INST_KEY_MENU(ans_certfail_keymenu, ans_certfail_keys);
     SCROLL_S sargs;
     STORE_S *store;
-    int      the_answer = 'n', indent, len;
-    char     buf[500];
+    int      the_answer = 'n', indent, len, cols;
+    char     buf[500], buf2[500];
     char    *folded;
     char    *hst = host ? host : "<unknown>";
     char    *rsn = reason ? reason : "<unknown>";
@@ -1544,8 +1713,7 @@ pine_sslfailure(host, reason, flags)
     int       ok_novalidate = 0, warned = 0;
 
 
-    dprint(1, (debugfile, "sslfailure: host=%s reason=%s\n",
-			   host ? host : "?", reason ? reason : "?"));
+    dprint(1, (debugfile, "sslfailure: host=%s reason=%s\n", hst, rsn));
 
     if(flags & NET_SILENT)
       return;
@@ -1565,17 +1733,20 @@ pine_sslfailure(host, reason, flags)
 	}
     }
 
+    cols = ps_global->ttyo ? ps_global->ttyo->screen_cols : 80;
+    cols--;
+
     if(!(store = so_get(CharStar, NULL, EDIT_ACCESS)))
       return;
 
     strncpy(buf, "There was an SSL/TLS failure for the server", sizeof(buf));
-    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    folded = fold(buf, cols, cols, 0, 0, "", "");
     so_puts(store, folded);
     fs_give((void **)&folded);
     so_puts(store, "\n");
 
-    if((len=strlen(hst)) <= ps_global->ttyo->screen_cols){
-	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+    if((len=strlen(hst)) <= cols){
+	if((indent=((cols-len)/2)) > 0)
 	  so_puts(store, repeat_char(indent, SPACE));
 
 	so_puts(store, hst);
@@ -1584,7 +1755,7 @@ pine_sslfailure(host, reason, flags)
     else{
 	strncpy(buf, hst, sizeof(buf));
 	buf[sizeof(buf)-1] = '\0';
-	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	folded = fold(buf, cols, cols, 0, 0, "", "");
 	so_puts(store, folded);
 	fs_give((void **)&folded);
     }
@@ -1592,13 +1763,13 @@ pine_sslfailure(host, reason, flags)
     so_puts(store, "\n");
 
     strncpy(buf, "The reason for the failure was", sizeof(buf));
-    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    folded = fold(buf, cols, cols, 0, 0, "", "");
     so_puts(store, folded);
     fs_give((void **)&folded);
     so_puts(store, "\n");
 
-    if((len=strlen(rsn)) <= ps_global->ttyo->screen_cols){
-	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+    if((len=strlen(rsn)) <= cols){
+	if((indent=((cols-len)/2)) > 0)
 	  so_puts(store, repeat_char(indent, SPACE));
 
 	so_puts(store, rsn);
@@ -1607,7 +1778,7 @@ pine_sslfailure(host, reason, flags)
     else{
 	strncpy(buf, rsn, sizeof(buf));
 	buf[sizeof(buf)-1] = '\0';
-	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	folded = fold(buf, cols, cols, 0, 0, "", "");
 	so_puts(store, folded);
 	fs_give((void **)&folded);
     }
@@ -1615,13 +1786,13 @@ pine_sslfailure(host, reason, flags)
     so_puts(store, "\n");
 
     strncpy(buf, "This is just an informational message. With the current setup, SSL/TLS will not work. If this error re-occurs every time you run Pine, your current setup is not compatible with the configuration of your mail server. You may want to add the option", sizeof(buf));
-    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    folded = fold(buf, cols, cols, 0, 0, "", "");
     so_puts(store, folded);
     fs_give((void **)&folded);
     so_puts(store, "\n");
 
-    if((len=strlen(notls)) <= ps_global->ttyo->screen_cols){
-	if((indent=((ps_global->ttyo->screen_cols-len)/2)) > 0)
+    if((len=strlen(notls)) <= cols){
+	if((indent=((cols-len)/2)) > 0)
 	  so_puts(store, repeat_char(indent, SPACE));
 
 	so_puts(store, notls);
@@ -1630,24 +1801,68 @@ pine_sslfailure(host, reason, flags)
     else{
 	strncpy(buf, notls, sizeof(buf));
 	buf[sizeof(buf)-1] = '\0';
-	folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+	folded = fold(buf, cols, cols, 0, 0, "", "");
 	so_puts(store, folded);
 	fs_give((void **)&folded);
     }
 
     so_puts(store, "\n");
 
-    strncpy(buf, "to the name of the mail server you are attempting to access.",
+    strncpy(buf, "to the name of the mail server you are attempting to access. In other words, wherever you see the characters",
 	    sizeof(buf));
-    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    folded = fold(buf, cols, cols, 0, 0, "", "");
     so_puts(store, folded);
     fs_give((void **)&folded);
     so_puts(store, "\n");
 
-    strncpy(buf, "Type RETURN to continue.", sizeof(buf));
-    folded = fold(buf, ps_global->ttyo->screen_cols, ps_global->ttyo->screen_cols, 0, 0, "", "");
+    if((len=strlen(hst)) <= cols){
+	if((indent=((cols-len)/2)) > 0)
+	  so_puts(store, repeat_char(indent, SPACE));
+
+	so_puts(store, hst);
+	so_puts(store, "\n");
+    }
+    else{
+	strncpy(buf, hst, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	folded = fold(buf, cols, cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
+
+    so_puts(store, "\n");
+
+    strncpy(buf, "in your configuration, replace those characters with",
+	    sizeof(buf));
+    folded = fold(buf, cols, cols, 0, 0, "", "");
     so_puts(store, folded);
     fs_give((void **)&folded);
+    so_puts(store, "\n");
+
+    sprintf(buf2, "%.100s%.100s", hst, notls);
+    if((len=strlen(buf2)) <= cols){
+	if((indent=((cols-len)/2)) > 0)
+	  so_puts(store, repeat_char(indent, SPACE));
+
+	so_puts(store, buf2);
+	so_puts(store, "\n");
+    }
+    else{
+	strncpy(buf, buf2, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+	folded = fold(buf, cols, cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
+
+    so_puts(store, "\n");
+
+    if(ps_global->ttyo){
+	strncpy(buf, "Type RETURN to continue.", sizeof(buf));
+	folded = fold(buf, cols, cols, 0, 0, "", "");
+	so_puts(store, folded);
+	fs_give((void **)&folded);
+    }
 
     memset(&sargs, 0, sizeof(SCROLL_S));
     sargs.text.text = so_text(store);
@@ -1661,7 +1876,45 @@ pine_sslfailure(host, reason, flags)
     /* don't want to re-enter c-client */
     sargs.quell_newmail = 1;
 
-    scrolltool(&sargs);
+    if(ps_global->ttyo)
+      scrolltool(&sargs);
+    else{
+	char **q, **qp;
+	char  *p;
+	unsigned char c;
+	int    cnt = 0;
+	
+	/*
+	 * The screen isn't initialized yet, which should mean that this
+	 * is the result of a -p argument. Display_args_err knows how to deal
+	 * with the uninitialized screen, so we mess with the data to get it
+	 * in shape for display_args_err. This is pretty hacky.
+	 */
+	
+	so_seek(store, 0L, 0);		/* rewind */
+	/* count the lines */
+	while(so_readc(&c, store))
+	  if(c == '\n')
+	    cnt++;
+	
+	qp = q = (char **)fs_get((cnt+1) * sizeof(char *));
+	memset(q, 0, (cnt+1) * sizeof(char *));
+
+	so_seek(store, 0L, 0);		/* rewind */
+	p = buf;
+	while(so_readc(&c, store)){
+	    if(c == '\n'){
+		*p = '\0';
+		*qp++ = cpystr(buf);
+		p = buf;
+	    }
+	    else
+	      *p++ = c;
+	}
+
+	display_args_err(NULL, q, 0);
+	free_list_array(&q);
+    }
 
     ps_global->mangled_screen = 1;
     ps_global->painted_body_on_startup = 0;
@@ -2110,7 +2363,14 @@ imap_get_passwd(m_list, passwd, user, hostlist, altflag)
 {
     MMLOGIN_S *l;
     
-    dprint(9, (debugfile, "imap_get_passwd: user=%s\n",user ? user : "(null)"));
+    dprint(9, (debugfile,
+	       "imap_get_passwd: checking user=%s alt=%d host=%s%s%s\n",
+	       user ? user : "(null)",
+	       altflag,
+	       hostlist->name ? hostlist->name : "",
+	       (hostlist->next && hostlist->next->name) ? ", " : "",
+	       (hostlist->next && hostlist->next->name) ? hostlist->next->name
+						        : ""));
     for(l = m_list; l; l = l->next)
       if(imap_same_host(l->hosts, hostlist)
 	 && *user
@@ -2118,7 +2378,9 @@ imap_get_passwd(m_list, passwd, user, hostlist, altflag)
 	 && l->altflag == altflag){
 	  strncpy(passwd, l->passwd, NETMAXPASSWD);
 	  passwd[NETMAXPASSWD-1] = '\0';
-	  dprint(9, (debugfile, "imap_get_passwd: got one: user=%s\n", user));
+	  dprint(9, (debugfile, "imap_get_passwd: match\n"));
+	  dprint(10, (debugfile, "imap_get_passwd: trying passwd=\"%s\"\n",
+		      passwd));
 	  return(TRUE);
       }
 
@@ -2168,7 +2430,7 @@ imap_set_passwd(l, passwd, user, hostlist, altflag, ok_novalidate, warned)
     if(!(*l)->user)
       (*l)->user = cpystr(user);
 
-    dprint(9, (debugfile, "saving user=%s altflag=%d\n", (*l)->user,
+    dprint(9, (debugfile, "imap_set_passwd: user=%s altflag=%d\n", (*l)->user,
 	       (*l)->altflag));
 
     for( ; hostlist; hostlist = hostlist->next){
@@ -2180,10 +2442,13 @@ imap_set_passwd(l, passwd, user, hostlist, altflag, ok_novalidate, warned)
 	if(!*listp){
 	    *listp = (STRLIST_S *)fs_get(sizeof(STRLIST_S));
 	    (*listp)->name = cpystr(hostlist->name);
-	    dprint(9, (debugfile, "host=%s\n", (*listp)->name));
+	    dprint(9, (debugfile, "imap_set_passwd: host=%s\n",
+		       (*listp)->name));
 	    (*listp)->next = NULL;
 	}
     }
+
+    dprint(10, (debugfile, "imap_set_passwd: passwd=\"%s\"\n", passwd));
 }
 
 
@@ -2262,6 +2527,7 @@ ps_get(size)
 
 static	int		xlate_key;
 static	MMLOGIN_S	*passfile_cache = NULL;
+static  int             using_passfile;
 
 
 
@@ -2311,26 +2577,34 @@ xlate_out(c)
 
 
 char *
-passfile_name(pinerc, path)
+passfile_name(pinerc, path, len)
     char *pinerc, *path;
+    size_t len;
 {
     struct stat  sbuf;
     char	*p = NULL;
     int		 i, j;
 
-    if(!path || !pinerc || pinerc[0] == '\0')
+    if(!path || !((pinerc && pinerc[0]) || ps_global->passfile))
       return(NULL);
 
-    if(p = last_cmpnt(pinerc))
-      for(i = 0; pinerc < p; pinerc++, i++)
-	path[i] = *pinerc;
-    else
-      i = 0;
+    if(ps_global->passfile)
+      strncpy(path, ps_global->passfile, len-1);
+    else{
+	if((p = last_cmpnt(pinerc)) && *(p-1) && *(p-1) != PASSFILE[0])
+	  for(i = 0; pinerc < p && i < len; pinerc++, i++)
+	    path[i] = *pinerc;
+	else
+	  i = 0;
 
-    for(j = 0; path[i] = PASSFILE[j]; i++, j++)
-      ;
+	for(j = 0; (i < len) && (path[i] = PASSFILE[j]); i++, j++)
+	  ;
+	
+    }
 
-    dprint(9, (debugfile, "Looking for passfile=%s\n", path));
+    path[len-1] = '\0';
+
+    dprint(9, (debugfile, "Looking for passfile \"%s\"\n", path));
 
 #if	defined(DOS) || defined(OS2)
     return((stat(path, &sbuf) == 0
@@ -2355,20 +2629,29 @@ passfile_name(pinerc, path)
 
 
 
+/*
+ * Passfile lines are
+ *
+ *   passwd TAB user TAB hostname TAB flags [ TAB orig_hostname ] \n
+ *
+ * In pine4.40 and before there was no orig_hostname, and there still isn't
+ * if it is the same as hostname.
+ */
 int
 read_passfile(pinerc, l)
     char       *pinerc;
     MMLOGIN_S **l;
 {
-    char  tmp[MAILTMPLEN], *ui[4];
+    char  tmp[MAILTMPLEN], *ui[5];
     int   i, j, n;
     FILE *fp;
 
     dprint(9, (debugfile, "read_passfile\n"));
     /* if there's no password to read, bag it!! */
-    if(!passfile_name(pinerc, tmp) || !(fp = fopen(tmp, "r")))
+    if(!passfile_name(pinerc, tmp, sizeof(tmp)) || !(fp = fopen(tmp, "r")))
       return(0);
 
+    using_passfile = 1;
     for(n = 0; fgets(tmp, sizeof(tmp), fp); n++){
 	/*** do any necessary DEcryption here ***/
 	xlate_key = n;
@@ -2378,8 +2661,9 @@ read_passfile(pinerc, l)
 	if(i && tmp[i-1] == '\n')
 	  tmp[i-1] = '\0';			/* blast '\n' */
 
-	ui[0] = ui[1] = ui[2] = ui[3] = NULL;
-	for(i = 0, j = 0; tmp[i] && j < 4; j++){
+	dprint(10, (debugfile, "read_passfile: %s\n", tmp));
+	ui[0] = ui[1] = ui[2] = ui[3] = ui[4] = NULL;
+	for(i = 0, j = 0; tmp[i] && j < 5; j++){
 	    for(ui[j] = &tmp[i]; tmp[i] && tmp[i] != '\t'; i++)
 	      ;					/* find end of data */
 
@@ -2389,11 +2673,19 @@ read_passfile(pinerc, l)
 
 	dprint(9, (debugfile, "read_passfile: calling imap_set_passwd\n"));
 	if(ui[0] && ui[1] && ui[2]){		/* valid field? */
-	    STRLIST_S hostlist[1];
+	    STRLIST_S hostlist[2];
 	    int	      flags = ui[3] ? atoi(ui[3]) : 0;
 
 	    hostlist[0].name = ui[2];
-	    hostlist[0].next = NULL;
+	    if(ui[4]){
+		hostlist[0].next = &hostlist[1];
+		hostlist[1].name = ui[4];
+		hostlist[1].next = NULL;
+	    }
+	    else{
+		hostlist[0].next = NULL;
+	    }
+
 	    imap_set_passwd(l, ui[0], ui[1], hostlist, flags & 0x01, 0, 0);
 	}
     }
@@ -2415,13 +2707,17 @@ write_passfile(pinerc, l)
 
     dprint(9, (debugfile, "write_passfile\n"));
     /* if there's no password to read, bag it!! */
-    if(!passfile_name(pinerc, tmp) || !(fp = fopen(tmp, "w")))
+    if(!passfile_name(pinerc, tmp, sizeof(tmp)) || !(fp = fopen(tmp, "w")))
       return;
 
     for(n = 0; l; l = l->next, n++){
 	/*** do any necessary ENcryption here ***/
-	sprintf(tmp, "%.100s\t%.100s\t%.100s\t%d\n", l->passwd, l->user,
-		l->hosts->name, l->altflag);
+	sprintf(tmp, "%.100s\t%.100s\t%.100s\t%d%s%s\n", l->passwd, l->user,
+		l->hosts->name, l->altflag,
+		(l->hosts->next && l->hosts->next->name) ? "\t" : "",
+		(l->hosts->next && l->hosts->next->name) ? l->hosts->next->name
+							 : "");
+	dprint(10, (debugfile, "write_passfile: %s", tmp));
 	xlate_key = n;
 	for(i = 0; tmp[i]; i++)
 	  tmp[i] = xlate_in(tmp[i]);
@@ -2445,13 +2741,18 @@ get_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
     STRLIST_S *hostlist;
     int	       altflag;
 {
+    dprint(9, (debugfile, "get_passfile_passwd\n"));
     return((passfile_cache || read_passfile(pinerc, &passfile_cache))
 	     ? imap_get_passwd(passfile_cache, passwd,
 			       user, hostlist, altflag)
 	     : 0);
 }
 
-
+int
+is_using_passfile()
+{
+    return(using_passfile ? 1 : 0);
+}
 
 /*
  * Just trying to guess the username the user might want to use on this
@@ -2480,6 +2781,7 @@ set_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
     STRLIST_S *hostlist;
     int	       altflag;
 {
+    dprint(9, (debugfile, "set_passfile_passwd\n"));
     if((passfile_cache || read_passfile(pinerc, &passfile_cache))
        && want_to("Preserve password on DISK for next login", 'y', 'x',
 		  NO_HELP, WT_NORM) == 'y'){
@@ -2487,4 +2789,40 @@ set_passfile_passwd(pinerc, passwd, user, hostlist, altflag)
 	write_passfile(pinerc, passfile_cache);
     }
 }
+
+
+/*
+ * Passfile lines are
+ *
+ *   passwd TAB user TAB hostname TAB flags [ TAB orig_hostname ] \n
+ *
+ * In pine4.40 and before there was no orig_hostname.
+ * This routine attempts to repair that.
+ */
+void
+update_passfile_hostlist(pinerc, user, hostlist, altflag)
+    char      *pinerc;
+    char      *user;
+    STRLIST_S *hostlist;
+    int        altflag;
+{
+    MMLOGIN_S *l;
+    STRLIST_S *h1, *h2;
+    
+    for(l = passfile_cache; l; l = l->next)
+      if(imap_same_host(l->hosts, hostlist)
+	 && *user
+	 && !strcmp(user, l->user)
+	 && l->altflag == altflag){
+	  break;
+      }
+    
+    if(l && l->hosts && hostlist && !l->hosts->next && hostlist->next &&
+       hostlist->next->name){
+	l->hosts->next = (STRLIST_S *)fs_get(sizeof(STRLIST_S));
+	l->hosts->next->name = cpystr(hostlist->next->name);
+	l->hosts->next->next = NULL;
+	write_passfile(pinerc, passfile_cache);
+    }
+}
 #endif	/* PASSFILE */
