@@ -1,4 +1,4 @@
-# $Id: makefile.hpp,v 4.44 2001/07/03 17:03:38 hubert Exp $
+# $Id: makefile.hpp,v 4.45 2001/10/23 22:24:09 hubert Exp $
 #
 #            T H E    P I N E    M A I L   S Y S T E M
 #
@@ -62,7 +62,7 @@ LDAPOFILES=   addrbook.o adrbkcmd.o args.o bldaddr.o init.o \
 
 STDLIBS=     -ltermcap
 LOCLIBS=     $(PICODIR)/libpico.a $(CCLIENTDIR)/c-client.a
-LIBS=        $(EXTRALIBES) $(LOCLIBS) $(LDAPLIBS) $(STDLIBS) \
+LIBS=        $(LOCLIBS) $(LDAPLIBS) $(STDLIBS) \
              `cat $(CCLIENTDIR)/LDFLAGS`
 
 STDCFLAGS=   -Dconst= -Aa -D_HPUX_SOURCE -DHPP -DSYSTYPE=\"HPP\" -DMOUSE
