@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.109 2000/11/13 20:39:39 hubert Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.113 2001/06/21 17:54:43 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: newmail.c,v 4.109 2000/11/13 20:39:39 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2000 by the University of Washington.
+   1989-2001 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -92,7 +92,6 @@ new_mail(force, time_for_check_point, flags)
     time_t        now;
     long          n, rv = 0;
     MAILSTREAM   *stream;
-    struct variable *vars = ps_global->vars;
     register struct pine *pine_state;
     int           checknow = 0;
 
@@ -225,7 +224,8 @@ new_mail(force, time_for_check_point, flags)
 	    || pine_state->unsorted_newmail)
 	   && !((flags & NM_DEFER_SORT)
 		|| any_lflagged(pine_state->msgmap, MN_HIDE)))
-	  refresh_sort(pine_state->msgmap, (flags & NM_STATUS_MSG));
+	  refresh_sort(pine_state->msgmap,
+		       (flags & NM_STATUS_MSG) ? SRT_VRB : SRT_NON);
 	else if(pine_state->new_mail_count > 0L)
 	  pine_state->unsorted_newmail = 1;
 
@@ -246,6 +246,9 @@ new_mail(force, time_for_check_point, flags)
 			 mailbox_mail_since_command, n);
 	    }
         }
+
+	if(flags & NM_STATUS_MSG)
+	  pine_state->mail_box_changed = 0;
     }
 
     if(pine_state->inbox_changed
@@ -285,13 +288,12 @@ new_mail(force, time_for_check_point, flags)
 			    inbox_mail_since_command, n);
 	    }
         }
+
+	if(flags & NM_STATUS_MSG)
+	  pine_state->inbox_changed = 0;
     }
 
     rv += pine_state->expunge_count;
-    if(flags & NM_STATUS_MSG){
-	pine_state->inbox_changed    = 0;
-	pine_state->mail_box_changed = 0;
-    }
 
     dprint(6, (debugfile, "******** new mail returning %ld  ********\n", 
 	       rv ? rv : -1));
@@ -318,7 +320,7 @@ new_mail_mess(stream, folder, number, max_num)
      char       *folder;
 {
     ENVELOPE	*e;
-    char	*subject = NULL, *from = NULL, tmp[MAILTMPLEN],
+    char	*subject = NULL, *from = NULL, tmp[MAILTMPLEN+1],
 		 intro[MAX_SCREEN_COLS+1], subj_leadin[MAILTMPLEN];
     static char *carray[] = { "regarding",
 				"concerning",
@@ -338,8 +340,12 @@ new_mail_mess(stream, folder, number, max_num)
 
     if(e && e->from){
         if(e->from->personal && e->from->personal[0]){
-	    /* "75" gets us the first valid MIME3 chunk */
-	    sprintf(tmp, "%.75s", e->from->personal);
+	    /*
+	     * The reason we use so many characters for tmp is because we
+	     * may have multiple MIME3 chunks and we don't want to truncate
+	     * in the middle of one of them before decoding.
+	     */
+	    sprintf(tmp, "%.*s", MAILTMPLEN, e->from->personal);
  	    from = cpystr((char *) rfc1522_decode((unsigned char *)tmp_20k_buf,
  						  SIZEOF_20KBUF, tmp, NULL));
  	}
@@ -354,10 +360,11 @@ new_mail_mess(stream, folder, number, max_num)
 
     if(number <= 1L) {
         if(e && e->subject && e->subject[0]){
- 	    sprintf(tmp, "%.75s", e->subject);
+ 	    sprintf(tmp, "%.*s", MAILTMPLEN, e->subject);
  	    subject = cpystr((char *) rfc1522_decode((unsigned char *)tmp_20k_buf,
  						     SIZEOF_20KBUF, tmp, NULL));
 	}
+
 	sprintf(subj_leadin, " %s ", carray[(unsigned)random()%12]);
         if(!from)
           subj_leadin[1] = toupper((unsigned char)subj_leadin[1]);
@@ -396,7 +403,7 @@ new_mail_mess(stream, folder, number, max_num)
     }
 
     q_status_message7(SM_ASYNC | SM_DING, 0, 60,
- 		      "%s%s%s%s%s%s%s", intro,
+ 		      "%s%s%s%.80s%s%.80s%s", intro,
  		      from ? ((number > 1L) ? " Most recent f" : " F") : "",
  		      from ? "rom " : "",
  		      from ? from : "",
@@ -408,7 +415,7 @@ new_mail_mess(stream, folder, number, max_num)
  		      (number <= 1L) ? (subject[0] ? "" : "ith no subject")
 				     : "");
 
-    sprintf(tmp_20k_buf, "%s%s%s%s", intro,
+    sprintf(tmp_20k_buf, "%s%s%s%.80s", intro,
 	    from ? ((number > 1L) ? " Most recent f" : " F") : "",
 	    from ? "rom " : "",
 	    from ? from : "");
@@ -507,8 +514,6 @@ check_point(timing, flags)
     int     freq, tm;
     long    adj_cca;
     long    tmp;
-    COLOR_PAIR *lastc = NULL, *newc;
-    struct variable *vars = ps_global->vars;
 #ifdef	DEBUG
     time_t  now;
 #endif
