@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.133 2001/06/21 17:54:43 hubert Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.135 2001/11/01 19:19:44 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -321,9 +321,7 @@ int		 row,
 		    Write_to_screen(temp);
 		    c = k->column + strlen(temp);
 		    if(!empty){
-			if(name_color)
-			  (void)pico_set_colorp(label_color, PSC_NONE);
-			else
+			if(!name_color)
 			  EndInverse();
 		    }
 
@@ -358,6 +356,9 @@ int		 row,
 		    }
 
 		    strcpy(last_time->label, this_label);
+		    if(label_color)
+		      (void)pico_set_colorp(label_color, PSC_NONE);
+
 		    Write_to_screen(temp);
 		}
 	    }
@@ -1519,6 +1520,11 @@ update_titlebar_status()
     if(!as.stream || as.current_msg <= 0L || as.del_column < 0)
       return(1);
 
+    if(as.current_msg != mn_get_cur(as.msgmap)){
+	update_titlebar_message();
+	return(1);
+    }
+
     mc = mail_elt(as.stream, mn_m2raw(as.msgmap, as.current_msg));
 
     if(!mc->valid)
