@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.116 1999/07/07 23:40:19 hubert Exp $";
+static char rcsid[] = "$Id: status.c,v 4.121 2000/11/29 17:22:06 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: status.c,v 4.116 1999/07/07 23:40:19 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1999 by the University of Washington.
+   1989-2000 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -64,6 +64,7 @@ typedef struct message {
 
 #define	LAST_MESSAGE(X)	((X) == (X)->next)
 #define	RAD_BUT_COL	0
+#define WANT_TO_BUF     256
 
 
 /*
@@ -90,6 +91,7 @@ INST_KEY_MENU(modal_message_keymenu, modal_message_keys);
  * Internal prototypes
  */
 int  output_message PROTO((SMQ_T *));
+int pre_screen_config_want_to PROTO((char *, int, int));
 void radio_help PROTO((int, int, HelpType));
 void draw_radio_prompt PROTO((int, int, int, char *));
 void pause_for_current_message PROTO(());
@@ -903,7 +905,7 @@ output_message(mq_entry)
 		    sprintf(t, "%*.*s\n", indent + p - m->text, p - m->text, m->text);
 		    t += strlen(t);
 
-		    for(p += 7; *p && isspace(*p); p++)
+		    for(p += 7; *p && isspace((unsigned char)*p); p++)
 		      ;
 		    indent += 8;
 		}
@@ -913,7 +915,7 @@ output_message(mq_entry)
 		
 		while(strlen(p) > ps_global->ttyo->screen_cols - 2 * indent){
 		    for(q = p + ps_global->ttyo->screen_cols - 2 * indent; 
-			q > p && !isspace(*q); q--)
+			q > p && !isspace((unsigned char)*q); q--)
 		      ;
 
 		    sprintf(t, "\n%*.*s", indent + q - p, q - p, p);
@@ -966,22 +968,26 @@ void
 delay_cmd_cue(on)
     int on;
 {
-    int l;
+    int l, screen_edge = 0;
     COLOR_PAIR *lastc;
     struct variable *vars = ps_global->vars;
 
     if(prevstartcol >= 0 && (l = strlen(prevstatusbuff))){
+	screen_edge = (prevstartcol == 0) || 
+	  (prevstartcol + l >= ps_global->ttyo->screen_cols);
 	MoveCursor(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global),
 		   prevstartcol ? max(prevstartcol - 1, 0) : 0);
 	lastc = pico_set_colors(VAR_STATUS_FORE_COLOR, VAR_STATUS_BACK_COLOR,
 				PSC_REV|PSC_RET);
 
-	Write_to_screen(on ? "[>" : " [");
+	Write_to_screen(on ? (screen_edge ? ">" : "[>") : 
+			(screen_edge ? "[" : " ["));
 
 	MoveCursor(ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global),
 		   min(prevstartcol + l, ps_global->ttyo->screen_cols) - 1);
 
-	Write_to_screen(on ? "<]" : "] ");
+	Write_to_screen(on ? (screen_edge ? "<" : "<]") : 
+			(screen_edge ? "]" : "] "));
 
 	if(lastc){
 	    (void)pico_set_colorp(lastc, PSC_NONE);
@@ -1008,8 +1014,8 @@ modal_bogus_input(ch)
 {
     char s[MAX_SCREEN_COLS+1];
 		
-    sprintf(s, "Command \"%s\" not allowed.  Press RETURN to continue Pine.",
-	    pretty_command(ch));
+    sprintf(s, "Command \"%.*s\" not allowed.  Press RETURN to continue Pine.",
+	    sizeof(s)-70, pretty_command(ch));
     status_message_write(s, 0);
     Writechar(BELL, 0);
     return(0);
@@ -1027,6 +1033,48 @@ static ESCKEY_S yorn[] = {
     {-1, 0, NULL, NULL}
 };
 
+int
+pre_screen_config_want_to(question, dflt, on_ctrl_C)
+     char    *question;
+     int      dflt, on_ctrl_C;
+{
+    int ret = 0;
+    char rep[WANT_TO_BUF], *p;
+#ifdef _WINDOWS
+    rep[0] = '\0';
+    mswin_flush();
+    if(strlen(question) + 3 < WANT_TO_BUF)
+      sprintf(rep, "%s ?", question);
+    switch (mswin_yesno (*rep ? rep : question)) {
+      default:
+      case 0:   return (on_ctrl_C);
+      case 1:   return ('y');
+      case 2:   return ('n');
+    }
+#endif
+    while(!ret){
+      fprintf(stdout, "%s? [%c]:", question, dflt);
+      fgets(rep, WANT_TO_BUF, stdin);
+      if((p = strpbrk(rep, "\r\n")) != NULL)
+	*p = '\0';
+      switch(*rep){
+        case 'Y':
+        case 'y':
+	  ret = (int)'y';
+	  break;
+        case 'N':
+        case 'n':
+	  ret = (int)'n';
+	  break;
+        case '\0':
+	  ret = dflt;
+	  break;
+        default:
+	  break;
+      }
+    }
+    return ret;
+}
 
 /*----------------------------------------------------------------------
      Ask a yes/no question in the status line
@@ -1055,6 +1103,8 @@ want_to(question, dflt, on_ctrl_C, help, flags)
     char *q2;
     int	  rv;
 
+    if(!ps_global->ttyo)
+      return(pre_screen_config_want_to(question, dflt, on_ctrl_C));
 #ifdef _WINDOWS
     if (mswin_usedialog ()) {
 	mswin_flush ();
